//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <clipbrd.hpp>
#include <inifiles.hpp>

#include "cfgmsgdlgs.h"
#include "qsadbs.h"
#include "global.h"
#include "utility.h"
#include "gnupg.h"
#include "gpg4.h"
#include "badcommadlgs.h"
#include "messagepipes.h"
#include "help.h"
#include "nymoptsdlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TCfgMsgDlg *CfgMsgDlg;
const AnsiString defnymoptions = "+acksend +cryptrecv -nobcc -signsend -fingerkey";
const AnsiString defmail2news = "mail2news@dizum.com,mail2news@m2n.mixmin.net";
//---------------------------------------------------------------------------

void __fastcall ManageNymConfigMessages(TForm *Owner)
{
   TCfgMsgDlg *cfgdlg = new TCfgMsgDlg(Owner);
   cfgdlg->ShowModal();
   delete cfgdlg;
}
//---------------------------------------------------------------------------

__fastcall TCfgMsgDlg::TCfgMsgDlg(TComponent* Owner)
	: TForm(Owner)
{
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(ListView->Font, ini->ReadString("settings", "windowfont", OptsDefWindowFont));
   maskgpgver = ini->ReadBool("settings", "maskgpgversion", false);
   delete ini;

   TStringList *sub = new TStringList();
   TStringList *subs = new TStringList();
   DB->LoadSubjects(subs);
   for (int i = 0; i < subs->Count; i++)
   {
	  sub->CommaText = subs->Strings[i];
	  if (ReadBool(sub, "active", false))
	  {
		 AnsiString keyid = ReadString(sub, "keyid", "");
		 if (keyid.IsEmpty())
			continue;

		 TListItem *item = ListView->Items->Add();
		 AnsiString nym, keystr = GPGGetPublicKeyStr(keyid);
		 if (keystr.IsEmpty())
		 {
			MsgBox((TForm*)Owner,
			  "I can't find a PGP key for key ID: "+keyid+"    \n\n"
			  "Have you possibly deleted a nym? If so, you may have       \n"
			  "deleted the nym's PGP key, but forgotten to delete the         \n"
			  "associated AAM subject line.",
				  MT_ERROR, MB_OK);
			continue;
		 }
		 if (keystr.Pos("<"))
		 {
			nym = GetTok(1, keystr, '<');
			nym = GetTok(0, nym, '>');
		 }
		 if (nym.IsEmpty())
		 {
			MsgBox((TForm*)Owner,
			  "I can't find a NYM for key ID: '"+keyid+"'",
			   MT_ERROR, MB_OK);
			continue;
		 }
		 //assert(nym != "");
		 item->Caption = nym;
		 item->SubItems->Add(subs->Strings[i]);
	  }
   }
   Label2->Caption = "";
   //look(subs);
   delete subs;
   delete sub;
}
/*__fastcall TCfgMsgDlg::TCfgMsgDlg(TComponent* Owner)
	: TForm(Owner)
{
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(ListView->Font, ini->ReadString("settings", "windowfont", OptsDefWindowFont));
   maskgpgver = ini->ReadBool("settings", "maskgpgversion", false);
   delete ini;

   TStringList *sub = new TStringList();
   TStringList *subs = new TStringList();
   DB->LoadSubjects(subs);
   for (int i = 0; i < subs->Count; i++)
   {
	  sub->CommaText = subs->Strings[i];
	  if (ReadBool(sub, "active", false))
	  {
		 AnsiString keyid = ReadString(sub, "keyid", "");
		 TListItem *item = ListView->Items->Add();
		 AnsiString nym, keystr = GPGGetPublicKeyStr(keyid);
		 if (keystr.Pos("<"))
		 {
			nym = GetTok(1, keystr, '<');
			nym = GetTok(0, nym, '>');
		 }
		 assert(nym != "");
		 item->Caption = nym;
		 item->SubItems->Add(subs->Strings[i]);
	  }
   }
   Label2->Caption = "";
   //look(subs);
   delete subs;
   delete sub;
}*/
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::ListViewClick(TObject *Sender)
{
   bool sel = ListView->Selected;
   CreateBtn->Enabled = sel;
   DeleteBtn->Enabled = sel;
   ChangeBtn->Enabled = sel;
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::WMShowStatus(TMessage &msg)
{
   Label2->Caption = (char*)msg.LParam;
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::CreateBtnClick(TObject *Sender)
{
   assert(ListView->Selected);
   if (MsgBox(this,
	  "Generate a nym create message now?   ",
	   MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
		  return;

   AnsiString keyid, type, nym=ListView->Selected->Caption, domain=GetDomain(nym);
   TStringList *sub = new TStringList();
   sub->CommaText = ListView->Selected->SubItems->Strings[0];
   assert(sub->Count);
   keyid = sub->Values["keyid"];
   type = DB->GetNymserverType(domain);
   assert(type != "");
   assert(nym.Pos("@"+domain) > 0);

   if (type.LowerCase() == "ghio")
	  GenerateGhioCreateNymConfigMessage(nym, sub);
   else
	  GenerateZaxCreateNymConfigMessage(nym, sub);

   delete sub;
   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::DeleteBtnClick(TObject *Sender)
{
   if (MsgBox(this,
	  "Generate a nym delete message now?   ",
	   MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
		  return;

   AnsiString type, nym=ListView->Selected->Caption, domain=GetDomain(nym);
   TStringList *sub = new TStringList();
   sub->CommaText = ListView->Selected->SubItems->Strings[0];
   assert(sub->Count);
   type = DB->GetNymserverType(domain);
   assert(type != "");
   assert(nym.Pos("@"+domain) > 0);

   if (type.LowerCase() == "ghio")
	  GenerateGhioDeleteNymConfigMessage(nym, sub);
   else
	  GenerateZaxDeleteNymConfigMessage(nym, sub);

   delete sub;
   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::ChangeBtnClick(TObject *Sender)
{
   if (MsgBox(this,
	  "Generate a nym key change message now?   ",
	   MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
		  return;

   AnsiString type, keyid, nym=ListView->Selected->Caption, domain=GetDomain(nym);
   TStringList *sub = new TStringList();
   sub->CommaText = ListView->Selected->SubItems->Strings[0];
   assert(sub->Count);
   keyid = sub->Values["keyid"];
   type = DB->GetNymserverType(domain);
   assert(type != "");
   assert(nym.Pos("@"+domain) > 0);

   if (type.LowerCase() == "ghio")
	  GenerateGhioChangeKeyNymConfigMessage(nym, sub);
   else
	  GenerateZaxChangeKeyNymConfigMessage(nym, sub);

   delete sub;
   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::ReportDefZaxNymserver(const AnsiString &domain)
{
   MsgBox(this,
	  "I have a problem. The nymserver, "+domain+", isn't listed in my   \n"
	  "qsa.servers.txt. For a temporary fix, I'm using a default section.   \n"
	  "I think it will work fine, but there might be an update for QSA now   \n"
	  "or in the near future. Please take a look.",
	   MT_INFO, MB_OK);
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::ReportDefGhioNymserver(const AnsiString &domain)
{
   MsgBox(this,
	  "I have a problem. The nymserver, "+domain+", isn't listed in my   \n"
	  "qsa.servers.txt. For a temporary fix, I'm using a default section.   \n"
	  "However, default may not work perfect. Possibly there is an update   \n"
	  "for QSA that lists this server. Please take a look.",
	   MT_INFO, MB_OK);
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::ReportBadNymserver(const AnsiString &domain)
{
   MsgBox(this,
	  "Problem! I can't create your config message. I don't have a   \n"
	  "section for this server,"+domain+", in my qsa.servers.txt.   \n"
	  "Also, the default section I would use instead is missing as   \n"
	  "well.\n\n"
	  "Reinstalling the program should correct the problem, but   \n"
	  "check for a QSA update. If this is a new nymserver it may   \n"
	  "have been added--or will soon.",
	   MT_ERROR, MB_OK);
}
//---------------------------------------------------------------------------

AnsiString __fastcall GetEncryptKey(const AnsiString &encrypttype)
{
   if (encrypttype.LowerCase() == "3des") return "Encrypt-3DES:";
   if (encrypttype.LowerCase() == "cast") return "Encrypt-CAST:";
   return "Encrypt-Key:";
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::GenerateGhioCreateNymConfigMessage(const AnsiString &nym,
   TStrings *subject)
{
   AnsiString fcc, domain=GetDomain(nym), serverkeystr=GPGGetPublicKeyStr("send@"+domain),
	  keyid=subject->Values["keyid"], serverkeyid, nymkeystr, thenym, nymname, nymcmds;
   if (serverkeystr.IsEmpty())
	  serverkeystr = GPGGetPublicKeyStr("config@"+domain);
   serverkeyid = GetTok(4, serverkeystr, ':');
   serverkeyid = "0x"+serverkeyid.SubString(serverkeyid.Length()-7, 8);
   assert(serverkeyid != "");
   nymkeystr = GPGGetPublicKeyStr(keyid);
   assert(nymkeystr != "");
   thenym = GetTok(0, nymkeystr, ':');
   int x = thenym.Pos("<");
   if (x)
	  nymname = thenym.SubString(1, x - 1).Trim();

   TStringList *pgpkey = new TStringList();
   if (!GPGExportKey(this, keyid, pgpkey, true))
   {
	  MsgBox(this,
		 "Sorry, I failed to get the required PGP key for " + nym + "!   ",
		  MT_ERROR, MB_OK);
	  delete pgpkey;
	  delete subject;
	  return;
   }
   if (maskgpgver)
	  pgpkey->Strings[1] = "Version: N/A";
   assert(pgpkey->Count);

   TStringList *server = new TStringList();
   DB->LoadNymserver(domain, server);

   // handle nym options and anon-to
   bool needsave = false;
   AnsiString substr, name, anonto, options;
   options = subject->Values["options"];
   anonto = subject->Values["anonto"];
   if (options.IsEmpty() || anonto.IsEmpty())
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  if (options.IsEmpty())
		 options = ini->ReadString("settings", "defoptions", defnymoptions);
	  if (anonto.IsEmpty())
		 anonto = ini->ReadString("settings", "defanonto", defmail2news);
	  needsave = true;
	  delete ini;
   }
   assert(options != "");
   assert(anonto != "");
   AnsiString organonto = anonto, orgoptions = options;
   if (!GetGhioNymOptions(this, thenym, anonto, options))
   {
	  delete pgpkey;
	  delete server;
	  return;
   }
   if (!needsave)
	  needsave = anonto != organonto || options != orgoptions;
   if (needsave)
   {
	  subject->Values["options"] = options;
	  subject->Values["anonto"] = anonto;
	  DB->SaveSubject(subject->Values["subno"], subject);
   }
   Snooze(200);

   TStringList *message = new TStringList();
   message->Add("Pgp: sign="+keyid+"; encrypt="+serverkeyid+";");
   message->Add("To: config@" + domain);
   message->Add("");
   message->Add("Config:");
   message->Add("From: " + nym);
   nymcmds = "create?" + options + " name=\"\"";
   x = nymcmds.Pos("name=\"");
   if (x && nymname != "")
   {
	  nymcmds.Insert(nymname, x + 6);
	  if (nymcmds.Length() > 50)
	  {
		 while (x > 50)
			x--;
		 while (x > 1 && nymcmds[x] != ' ')
			x--;
		 nymcmds.Insert("\r\n", x);
	  }
   }
   message->Add("Nym-Commands: " + nymcmds.TrimLeft());
   message->Add("Public-Key:");
   message->AddStrings(pgpkey);
   message->Add("Reply-Block:");
   message->Add("::");
   message->Add("Anon-To: " + anonto);
   if (ReadBool(subject, "esub", false))
	  message->Add("Encrypt-Subject: " + subject->Values["ephrase"]);
   else
	  message->Add("Hash-Subject: yes");

   if (ReadBool(subject, "ekey", false))
   {
	  AnsiString phrase = subject->Values["kphrase"],
		 encryptkey = GetEncryptKey(server->Values["enckey"]);
	  if (phrase.Pos(",") > 0)
	  {
		 ShowCommaError(this, encryptkey);
		 delete message;
		 delete server;
		 delete pgpkey;
		 return;
	  }
	  message->Add(encryptkey + " " + phrase);
   }
   message->Add("");
   message->Add("##");
   message->Add("Subject: " + subject->Values["subject"]);
   message->Add("Newsgroups: alt.anonymous.messages");
   message->Add("");
   message->Add("**");
   message->Add("~~");

   HandleMessageTransfer(this, message, QSCOPYDATA_NT);

   delete message;
   delete server;
   delete pgpkey;
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::GenerateGhioDeleteNymConfigMessage(const AnsiString &nym,
   TStrings *subject)
{
   AnsiString fcc, domain=GetDomain(nym), keyid=subject->Values["keyid"], serverkeyid, nymkeystr,
	  serverkeystr=GPGGetPublicKeyStr("send@"+domain);
   if (serverkeystr.IsEmpty())
	  serverkeystr = GPGGetPublicKeyStr("config@"+domain);
   serverkeyid = GetTok(4, serverkeystr, ':');
   serverkeyid = "0x"+serverkeyid.SubString(serverkeyid.Length()-7, 8);
   assert(serverkeyid != "");
   nymkeystr = GPGGetPublicKeyStr(keyid);
   assert(nymkeystr != "");

   TStringList *message = new TStringList();
   message->Add("Pgp: sign="+keyid+"; encrypt="+serverkeyid+";");
   message->Add("To: config@"+domain);
   message->Add("");
   message->Add("Config:");
   message->Add("From: " + nym);
   message->Add("Nym-Commands: delete");
   message->Add("~~");
   HandleMessageTransfer(this, message, QSCOPYDATA_NT);

   delete message;
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::GenerateZaxCreateNymConfigMessage(const AnsiString &nym,
   TStrings *subject)
{
   AnsiString fcc, domain=GetDomain(nym), serverkeystr=GPGGetPublicKeyStr("send@"+domain),
	  keyid=subject->Values["keyid"], serverkeyid;
   if (serverkeystr.IsEmpty())
	  serverkeystr = GPGGetPublicKeyStr("config@"+domain);
   serverkeyid = GetTok(4, serverkeystr, ':');
   serverkeyid = "0x"+serverkeyid.SubString(serverkeyid.Length()-7, 8);
   assert(serverkeyid != "");

   TStringList *pgpkey = new TStringList();
   if (!GPGExportKey(this, keyid, pgpkey, true))
   {
	  MsgBox(this,
		 "Sorry, I failed to get the required PGP key for " + nym + "!   ",
		  MT_ERROR, MB_OK);
	  delete pgpkey;
	  return;
   }
   assert(pgpkey->Count);
   if (maskgpgver)
	  pgpkey->Strings[1] = "Version: N/A";

   TStringList *message = new TStringList();
   message->Add("Pgp: sign="+keyid+"; encrypt="+serverkeyid+";");
   message->Add("To: config@"+domain);
   message->Add("");
   message->Add("hsub: " + subject->Values["subject"]);
   message->AddStrings(pgpkey);
   message->Add("");
   message->Add("~~");

   HandleMessageTransfer(this, message, QSCOPYDATA_NT);

   delete message;
   delete pgpkey;
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::GenerateZaxDeleteNymConfigMessage(const AnsiString &nym,
	TStrings *subject)
{
   AnsiString fcc, domain=GetDomain(nym), serverkeystr=GPGGetPublicKeyStr("send@"+domain),
	  keyid=subject->Values["keyid"], serverkeyid;
   if (serverkeystr.IsEmpty())
	  serverkeystr = GPGGetPublicKeyStr("config@"+domain);
   serverkeyid = GetTok(4, serverkeystr, ':');
   serverkeyid = "0x"+serverkeyid.SubString(serverkeyid.Length()-7, 8);
   assert(serverkeyid != "");

   TStringList *message = new TStringList();
   message->Add("Pgp: sign="+keyid+"; encrypt="+serverkeyid+";");
   message->Add("To: config@"+domain);
   message->Add("");
   message->Add("delete: yes");
   message->Add("~~");

   HandleMessageTransfer(this, message, QSCOPYDATA_NT);

   delete message;
   //delete server;
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::GenerateGhioChangeKeyNymConfigMessage(const AnsiString &nym,
	TStrings *subject)
{
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::GenerateZaxChangeKeyNymConfigMessage(const AnsiString &nym,
	TStrings *subject)
{
}
//---------------------------------------------------------------------------

void __fastcall TCfgMsgDlg::HelpBtnClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/nymcfgmsgs.htm>cfgmsghelp");
}
//---------------------------------------------------------------------------

