/*
*  dbpackers.cpp
*
*  Copyright (C) 2012 Richard Christman
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 3 of the License, or (at your
*  option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver AAM, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors
*  of QuickSilver AAM grant you additional permission to convey the
*  resulting work.
*
*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <stdio.h>

#include "dbpackers.h"
#include "qsadbs.h"
#include "utility.h"
#include "global.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
TDBPacker *DBPacker;
//---------------------------------------------------------------------------

void __fastcall PackDatabase(TForm *Owner)
{
   DBPacker = new TDBPacker(Owner);
   DBPacker->PackDatabase();
   delete DBPacker;
}
//---------------------------------------------------------------------------

__fastcall TDBPacker::TDBPacker(TForm *owner)
{
   Owner = owner;
   Log = new TStringList();
   Log->Clear();
   itemfails = 0;
   fails = 0;
}
//---------------------------------------------------------------------------

__fastcall TDBPacker::~TDBPacker()
{
   delete Log;
}
//---------------------------------------------------------------------------

void __fastcall TDBPacker::LogLAST(const AnsiString &msg)
{
   Log->Strings[Log->Count-1] =
	  Log->Strings[Log->Count-1] + msg;
}
//---------------------------------------------------------------------------

void __fastcall TDBPacker::LogADD(const AnsiString &msg)
{
   //Label1->Caption = msg;
   Log->Add(msg);
}

//---------------------------------------------------------------------------
// Pack Database
//---------------------------------------------------------------------------

bool __fastcall TDBPacker::PackItem(TDBindex *frIndex, TDBindex *toIndex,
   const AnsiString &key, int capfield)
{
   bool ok = true;
   AnsiString cap = GetTok(capfield, key, DBFSc);
   TDBbuffer *buf = new TDBbuffer();

   if (frIndex->GetBinary(key, buf) != DB_OK)
   {
	  LogADD("  Read Error: " + cap);
	  ok = false;
	  fails++;
   }

//if (key == "session") sm(buf->data);

   if (ok && toIndex->PutBinary(key, buf, false) != DB_OK)
   {
	  LogADD("  Write Error: " + cap);
	  ok = false;
	  fails++;
   }

   delete buf;
   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TDBPacker::DoPackItem(TDBindex *frIndex, TDBindex *toIndex,
   const AnsiString &key, int capfield)
{
   //Snooze(100);
   if (PackItem(frIndex, toIndex, key, capfield))
	  LogLAST("ok");
}
//---------------------------------------------------------------------------

void __fastcall TDBPacker::PackIndex(TDBindex *frParent, TDBindex *toParent,
   const AnsiString &path, int capfield)
{
   itemfails = 0;
   TDBindex *frIndex = new TDBindex(frParent->dbm, path, frParent);
   TDBindex *toIndex = new TDBindex(toParent->dbm, path, toParent);

   for (int i = 0; i < frIndex->Count; i++)
   {
	  AnsiString key = frIndex->Strings[i];
	  if (frIndex->IsIndexKey(i))
	  {
		 LogADD("   " + key.LowerCase() + "...");
		 DoPackIndex(frIndex, toIndex, key, capfield);
	  }
	  else
	  {
		 if (!PackItem(frIndex, toIndex, key, capfield))
			itemfails++;
	  }
	  Application->ProcessMessages();
   }
   toIndex->Save();

   delete frIndex;
   delete toIndex;

   fails += itemfails;
}
//---------------------------------------------------------------------------

void __fastcall TDBPacker::DoPackIndex(TDBindex *frParent, TDBindex *toParent,
   const AnsiString &path, int capfield)
{
   //Snooze(100);
   PackIndex(frParent, toParent, path, capfield);
   if (!itemfails)
	  LogLAST("ok");
}
//---------------------------------------------------------------------------
// PackDatabase
//---------------------------------------------------------------------------

bool __fastcall TDBPacker::PackDatabase()
{
   TQSADB *db = new TQSADB("");  // DB is InMemory
   assert(db != NULL);

   TDBindex *frMain, *toMain;
   frMain = DB->GetIndex(0);
   toMain = db->GetIndex(0);
   LogADD("packing database");

   LogADD("packing proxies...");
   DoPackItem(frMain, toMain, "proxies");

   LogADD("packing hosts...");
   DoPackItem(frMain, toMain, "hosts");

   LogADD("packing subjects...");
   DoPackItem(frMain, toMain, "subjects");

   LogADD("packing log...");
   DoPackItem(frMain, toMain, "log");

   LogADD("packing session...");
   DoPackItem(frMain, toMain, "session");

   LogADD("packing settings...");
   DoPackItem(frMain, toMain, "settings");

   LogADD("packing inbox...");
   DoPackIndex(frMain, toMain, "inbox");

   toMain->Save();

   if (fails)
   {
	  LogADD(IntToStr(fails) + " items failed");
   }
   else
   {
	  LogADD("Success!");
   }

   if (fails)
   {
	  if (MsgBox(Owner,
		 "You have " + IntToStr(fails) + " failed operations. \n"
		 "Do you wish to keep this new datafile?   ",
		  MT_CONFIRM, MB_YESNO) == IDNO)
	  {
		 goto finish;
	  }
   }

   if (DB->IsSecureMode())
   {
	  db->MasterPassword = new TMemoryStream();
	  db->MasterPassword->LoadFromStream(DB->MasterPassword);
	  //sm(db->MasterPassword->Size);
   }
   DB->modified = false;
   delete DB;
   if (FileExists(AppDBTmp()))
	  DeleteFile(AppDBTmp());
   DB = db;
   db = NULL;
   DB->modified = true;
   DB->filename = AppDBTmp();
   LogADD("saving packed database...");

   if (!DB->Save())
   {
	  MsgBox(Owner,
		"I failed to save packed database. I'm keeping the original!  \n"
		"The exception reported was: \n" +
		 DB->exceptmsg,
		 MT_ERROR, MB_OK);
   }

   LogLAST(FileExists(AppDBTmp()) ? "ok" : "FAILED");
   if (FileExists(AppDB()))
   {
	  LogADD("deleting original...");
	  DeleteFile(AppDB());
	  LogLAST(!FileExists(AppDB()) ? "ok" : "FAILED");
   }
   if (!FileExists(AppDB()))
	  rename(AppDBTmp().c_str(), AppDB().c_str());
   DB->filename = AppDB();
   LogADD("done");
   if (FileExists(AppDB()))
	  LogADD("OK");


finish:
   if (db)
   {
	  delete db;
   }
   return true;
}
//---------------------------------------------------------------------------
