//---------------------------------------------------------------------------
#pragma hdrstop
#include <clipbrd.hpp>

#include "messagepipes.h"
#include "global.h"
#include "utility.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
bool usepipedata = true;
//---------------------------------------------------------------------------

void __fastcall ShowStatus(TForm *Owner, const AnsiString &msg)
{  // this sendmessage uses 1 for wparam to tell the program not to delete after use
   SendMessage(Owner->Handle, WM_SHOWSTATUS, 1, (LPARAM)msg.c_str());
}
//---------------------------------------------------------------------------

void __fastcall HandleMessageTransfer(TForm *Owner, TStrings *message, WORD copytype)
{
//sm("Handle   "+AnsiString((int)Owner->Handle)+"   "+AnsiString((int)Application->MainForm->Handle));
   if (usepipedata && HasQSL())
   {
	  ShowStatus(Owner, "...processing message...");
	  if (!PipeMessageToQSL(Owner, message, copytype))
	  {
		 ShowStatus(Owner, " ");
		 Snooze(300);
		 if (MsgBox(Owner,
			"I failed to pipe the message to QSL. Sorry.  \n"
			"Would you like to use the clipboard?",
			 MT_ERROR, MB_YESNO) == IDYES)
		 {
			ShowStatus(Owner, "...copying message to clipboard...");
			Clipboard()->AsText = message->Text;
			Snooze(500);
		 }
	  }
   }
   else
   {
	  ShowStatus(Owner, "...copying message to clipboard...");
	  Clipboard()->AsText = message->Text;
	  if (HasQSL())
	  {
		 NotifyQSLClipboard(Owner,
			copytype == QSCOPYDATA_NT ? CLIPBOARDREPLY_NT : CLIPBOARDREPLY);
	  }
	  Snooze(500);
   }
   ShowStatus(Owner, " ");
}
//---------------------------------------------------------------------------

bool __fastcall PipeMessageToQSL(TForm *Owner, TStrings *msg, WORD copytype)
{
   if (!HasQSL())
	  return false;

   HWND hWnd = FindWindow("TQSLMainWin", NULL);
   if (!hWnd && FileExists(AppDir()+"qsl.exe"))
   {
	  Exec32(AppDir()+"qsl.exe -nowin");
	  for (int i = 0; i < 10; i++)
	  {
		  Snooze(1000);
		  hWnd = FindWindow("TQSLMainWin", NULL);
		  if (hWnd)
			 break;
	  }
	  if (!hWnd)
	  {
		 MsgBox(Owner, "Sorry, I failed to start QSL.   ", MT_ERROR, MB_OK);
		 return false;
	  }
   }

   AnsiString qsapipename = "\\\\.\\pipe\\qsa"+AnsiString(GetTickCount());
   HANDLE hPipe = CreateNamedPipe(
	  qsapipename.c_str(),          // pipe name
	  PIPE_ACCESS_OUTBOUND,         // read/write access
	  PIPE_TYPE_BYTE |              // message type pipe
	  PIPE_READMODE_BYTE |          // message-read mode
	  PIPE_WAIT,                    // blocking mode
	  1,                            // max. instances
	  1024,                         // output buffer size
	  1024,                         // input buffer size
	  NMPWAIT_USE_DEFAULT_WAIT,     // client time-out
	  NULL                          // default security attribute
   );
   if (hPipe == INVALID_HANDLE_VALUE)
   {
	  MsgBox(Owner,
		 "I failed to open a data pipe to QSL. The error returned was:   \n\n"+
		  GetLastErrorStr(),
		  MT_ERROR, MB_OK);
	  return false;
   }

   COPYDATASTRUCT cdata;
   unsigned long len = qsapipename.Length()+ 1;
   char *pipename = new char[len + 1];
   strncpy(pipename, qsapipename.c_str(), len);
   cdata.dwData = copytype;
   cdata.cbData = len;
   cdata.lpData = pipename;
   if (!SendMessage(hWnd, WM_COPYDATA, (WPARAM)Owner->Handle, (LPARAM)&cdata))
   {
	  MsgBox(Owner,
		 "I failed to send QSL the pipe name for data transfer!   ",
		  MT_ERROR, MB_OK);
	  CloseHandle(hPipe);
	  delete [] pipename;
	  return false;
   }

   bool connected = ConnectNamedPipe(hPipe, NULL);
   if (!connected)
   {
	  MsgBox(Owner,
		 "I failed to connect with QSL for data transfer!   ",
		  MT_ERROR, MB_OK);
	  CloseHandle(hPipe);
	  delete [] pipename;
	  return false;
   }

   unsigned long byteswritten = 0;
   len = msg->Text.Length();
   bool success = WriteFile(
	  hPipe,                // handle to pipe
	  msg->Text.c_str(),    // buffer to write from
	  len + 1,              // number of bytes to write, include the NULL
	  &byteswritten,        // number of bytes written
	  NULL);                // not overlapped I/O
   if (!success || byteswritten != len + 1)
   {
	  MsgBox(Owner,
		 "I failed piping data to QSL. The error reported was:   \n\n"+
		  GetLastErrorStr(),
		  MT_ERROR,
		  MB_OK);
	  CloseHandle(hPipe);
	  delete [] pipename;
	  return false;
   }

   //Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"...message piped to QSL...");
   Snooze(300);
   FlushFileBuffers(hPipe);
   CloseHandle(hPipe);
   delete [] pipename;
   return true;
}
//---------------------------------------------------------------------------

void __fastcall NotifyQSLClipboard(TForm *Owner, WORD cliptype)
{
   HWND hWnd = FindWindow("TQSLMainWin", NULL);
   if (!hWnd && HasQSL())
   {
	  Exec32(AppDir()+"qsl.exe -nowin");
	  for (int i = 0; i < 10; i++)
	  {
		  Snooze(1000);
		  hWnd = FindWindow("TQSLMainWin", NULL);
		  if (hWnd)
			 break;
	  }
   }
   if (!hWnd)
   {
	  MsgBox(Owner, "Sorry, I failed to start QSL.   ", MT_ERROR, MB_OK);
	  return;
   }

   Snooze(300);
   PostMessage(hWnd, WM_QSLMESSAGE, cliptype, 0);
}
//---------------------------------------------------------------------------

/*bool __fastcall PipeDataOrClipboardToQSLNoTemplate(TForm *Owner, TStrings *msg)
{
   HWND hWnd = FindWindow("TQSLMainWin",NULL);
   if (!hWnd && FileExists(AppDir()+"qsl.exe"))
   {
	  Exec32(AppDir()+"qsl.exe -nowin");
	  for (int i = 0; i < 5; i++)
	  {
		  Snooze(1000);
		  hWnd = FindWindow("TQSLMainWin",NULL);
		  if (hWnd)
			 break;
	  }
	  if (!hWnd)
	  {
		 MsgBox(Owner, "Sorry, I failed to start QSL.   ", MT_ERROR, MB_OK);
		 return false;
	  }
   }

   AnsiString qsapipename = "\\\\.\\pipe\\qsa"+AnsiString(GetTickCount());
   HANDLE hPipe = CreateNamedPipe(
	  qsapipename.c_str(),          // pipe name
	  PIPE_ACCESS_OUTBOUND,         // read/write access
	  PIPE_TYPE_BYTE |              // message type pipe
	  PIPE_READMODE_BYTE |          // message-read mode
	  PIPE_WAIT,                    // blocking mode
	  1,                            // max. instances
	  1024,                         // output buffer size
	  1024,                         // input buffer size
	  NMPWAIT_USE_DEFAULT_WAIT,     // client time-out
	  NULL                          // default security attribute
   );
   if (hPipe == INVALID_HANDLE_VALUE)
   {
	  MsgBox(Owner,
		 "I failed to open a data pipe to QSL. The error returned was:   \n\n"+
		  GetLastErrorStr(),
		  MT_ERROR, MB_OK);
	  return false;
   }

   COPYDATASTRUCT cdata;
   unsigned long len = qsapipename.Length()+ 1;
   char *pipename = new char[len + 1];
   strncpy(pipename, qsapipename.c_str(), len);
   cdata.dwData = QSCOPYDATANOTEMPLATE;
   cdata.cbData = len;
   cdata.lpData = pipename;
   if (!SendMessage(hWnd, WM_COPYDATA, (WPARAM)Owner->Handle, (LPARAM)&cdata))
   {
	  MsgBox(Owner,
		 "I failed to send QSL the pipe name for data transfer!   ",
		  MT_ERROR, MB_OK);
	  CloseHandle(hPipe);
	  delete [] pipename;
	  return false;
   }

   bool connected = ConnectNamedPipe(hPipe, NULL);
   if (!connected)
   {
	  MsgBox(Owner,
		 "I failed to connect with QSL for data transfer!   ",
		  MT_ERROR, MB_OK);
	  CloseHandle(hPipe);
	  delete [] pipename;
	  return false;
   }

   unsigned long byteswritten;

   len = msg->Text.Length();
   bool success = WriteFile(
	  hPipe,                // handle to pipe
	  msg->Text.c_str(),    // buffer to write from
	  len + 1,              // number of bytes to write, include the NULL
	  &byteswritten,        // number of bytes written
	  NULL);                // not overlapped I/O

   if (!success || byteswritten != len + 1)
   {
	  MsgBox(Owner,
		 "I failed piping data to QSL. The error reported was:   \n\n"+
		  GetLastErrorStr(),
		  MT_ERROR,
		  MB_OK);
	  CloseHandle(hPipe);
	  delete [] pipename;
	  return false;
   }

   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"Message piped to QSL...");
   FlushFileBuffers(hPipe);
   CloseHandle(hPipe);
   delete [] pipename;
look(msg);
   return true;
}*/
//---------------------------------------------------------------------------

bool __fastcall CheckNotifyQSLPipeDataOrClipboard(TForm *Owner, TStrings *msg)
{
   HWND hWnd = FindWindow("TQSLMainWin",NULL);
   if (!FileExists(AppDir()+"qsl.exe"))
   {
	  return false;
   }
   if (!hWnd && FileExists(AppDir()+"qsl.exe"))
   {
	  Exec32(AppDir()+"qsl.exe -nowin");
	  for (int i = 0; i < 5; i++)
	  {
		  Snooze(1000);
		  hWnd = FindWindow("TQSLMainWin",NULL);
		  if (hWnd)
			 break;
	  }
	  if (!hWnd)
	  {
		 MsgBox(Owner, "Sorry, I failed to start QSL.   ", MT_ERROR, MB_OK);
		 return false;
	  }
   }

   AnsiString qsapipename = "\\\\.\\pipe\\qsa"+AnsiString(GetTickCount());
   HANDLE hPipe = CreateNamedPipe(
	  qsapipename.c_str(),          // pipe name
	  PIPE_ACCESS_OUTBOUND,         // read/write access
	  PIPE_TYPE_BYTE |              // message type pipe
	  PIPE_READMODE_BYTE |          // message-read mode
	  PIPE_WAIT,                    // blocking mode
	  1,                            // max. instances
	  1024,                         // output buffer size
	  1024,                         // input buffer size
	  NMPWAIT_USE_DEFAULT_WAIT,     // client time-out
	  NULL                          // default security attribute
   );
   if (hPipe == INVALID_HANDLE_VALUE)
   {
	  MsgBox(Owner,
		 "I failed to open a data pipe to QSL. The error returned was:   \n\n"+
		  GetLastErrorStr(),
		  MT_ERROR, MB_OK);
	  return false;
   }

   COPYDATASTRUCT cdata;
   unsigned long len = qsapipename.Length()+ 1;
   char *pipename = new char[len + 1];
   strncpy(pipename, qsapipename.c_str(), len);
   cdata.dwData = QSCOPYDATA;
   cdata.cbData = len;
   cdata.lpData = pipename;
   if (!SendMessage(hWnd, WM_COPYDATA, (WPARAM)Owner->Handle, (LPARAM)&cdata))
   {
	  MsgBox(Owner,
		 "I failed to send QSL the pipe name for data transfer!   ",
		  MT_ERROR, MB_OK);
	  CloseHandle(hPipe);
	  delete [] pipename;
	  return false;
   }

   bool connected = ConnectNamedPipe(hPipe, NULL);
   if (!connected)
   {
	  MsgBox(Owner,
		 "I failed to connect with QSL for data transfer!   ",
		  MT_ERROR, MB_OK);
	  CloseHandle(hPipe);
	  delete [] pipename;
	  return false;
   }

   unsigned long byteswritten;
   len = msg->Text.Length();
   bool success = WriteFile(
	  hPipe,                // handle to pipe
	  msg->Text.c_str(),    // buffer to write from
	  len + 1,              // number of bytes to write, include the NULL
	  &byteswritten,        // number of bytes written
	  NULL);                // not overlapped I/O

   if (!success || byteswritten != len + 1)
   {
	  MsgBox(Owner,
		 "I failed piping data to QSL. The error reported was:   \n\n"+
		  GetLastErrorStr(),
		  MT_ERROR,
		  MB_OK);
	  CloseHandle(hPipe);
	  delete [] pipename;
	  return false;
   }

   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"Message piped to QSL...");
   FlushFileBuffers(hPipe);
   CloseHandle(hPipe);
   delete [] pipename;

   return true;
}
//---------------------------------------------------------------------------
