/*
*  pgpdlgs.cpp 
*
*  Copyright (C) 2012 Richard Christman
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors
*  of QuickSilver AAM grant you additional permission to convey the
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "pgpdlgs.h"
#include "global.h"
#include "utility.h"
#include "gnupg.h"
#include "gpg4.h"
#include "dbs.h"
#include "pgpdpassdlgs.h"
#include "msgmans.h"
#include "pgpcache.h"
#include "dsadlgs.h"
#include "qsadbs.h"
#include "blenddlgs.h"
#include "sigfaileddlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPgpDlg *PgpDlg;
const modeDecrypt = 0;
const modeDecryptQuiet = 1;
const modeDecryptAll = 2;
const modeSymmetric = 3;
const modeDecryptNym = 4;
//---------------------------------------------------------------------------

bool __fastcall DecryptNymMessage(TForm *Owner, TStrings *lines)
{
   AnsiString subno = lines->Strings[0];
   lines->Delete(0);
   TPgpDlg *pgpdlg = new TPgpDlg(Owner);
   pgpdlg->passphrase = passphrase;
   pgpdlg->lines->AddStrings(lines);
   pgpdlg->mode = modeDecryptNym;
   pgpdlg->subno = subno;
   pgpdlg->ShowModal();
   bool ok = pgpdlg->ok;
   //if (ok)
   {
	  lines->Clear();
	  lines->AddStrings(pgpdlg->lines);
   }
   delete pgpdlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall DecryptMessage(TForm *Owner, TStrings *lines)
{
   //AnsiString subno = lines->Strings[0];
   //lines->Delete(0);

   TPgpDlg *pgpdlg = new TPgpDlg(Owner);
   pgpdlg->passphrase = passphrase;
   pgpdlg->lines->AddStrings(lines);
   pgpdlg->mode = modeDecrypt;
   //pgpdlg->subno = subno;
   pgpdlg->ShowModal();
   bool ok = pgpdlg->ok;
   //if (ok)
   {
	  lines->Clear();
	  lines->AddStrings(pgpdlg->lines);
   }
   delete pgpdlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall DecryptSymmetric(TForm *Owner, TStrings *lines,
   AnsiString &passphrase)
{
   TPgpDlg *pgpdlg = new TPgpDlg(Owner);
   pgpdlg->passphrase = passphrase;
   pgpdlg->lines->AddStrings(lines);
   pgpdlg->mode = modeSymmetric;
   pgpdlg->Timer1->Interval = 10;
   pgpdlg->ShowModal();
   bool ok = pgpdlg->ok;
   //if (ok)
   {
	  lines->Clear();
	  lines->AddStrings(pgpdlg->lines);
   }
   delete pgpdlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TPgpDlg::TPgpDlg(TComponent* Owner)
	: TForm(Owner)
{
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   delete ini;

   PopupParent = OwnerForm();
   lines = new TStringList();
   ok = false;
   //ShowStatus("");
}
//---------------------------------------------------------------------------

__fastcall TPgpDlg::~TPgpDlg()
{
   delete lines;
}
//---------------------------------------------------------------------------

void __fastcall TPgpDlg::FormShow(TObject *Sender)
{
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TPgpDlg::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   //Snooze(100);
   switch (mode)
   {
	  case modeDecrypt:
		 ok = DecryptMessage(lines);
		 Snooze(500);
		 break;
	  case modeDecryptNym:
		 ok = DecryptNymMessage(lines);
		 Snooze(500);
		 break;
	  case modeSymmetric:
		 ok = DecryptSymmetric();
		 Snooze(500);
		 break;
   }
   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TPgpDlg::ShowStatus(const AnsiString &msg)
{
   Label1->Caption = " "+msg+" ";
}
//---------------------------------------------------------------------------

AnsiString __fastcall GetSubject(TStrings *msg)
{
   int x = 0;
   AnsiString subject;
   for (int i = 0; i < msg->Count; i++)
   {
	  AnsiString line = msg->Strings[i];
	  if (line.Pos("Subject:") == 1 && ++x == 2)
	  {
		 subject = line.SubString(9, line.Length()).TrimLeft();
		 break;
	  }
   }
   return subject;
}
//---------------------------------------------------------------------------

AnsiString __fastcall GetFrom(TStrings *msg)
{
   int x = 0;
   AnsiString from;
   for (int i = 0; i < msg->Count; i++)
   {
	  AnsiString line = msg->Strings[i];
	  if (line.Pos("From:") == 1) // && ++x == 2)
	  {
		 from = line.SubString(6, line.Length()).TrimLeft();
		 if (++x == 2)
			break;
	  }
   }
   return from;
}
//---------------------------------------------------------------------------

AnsiString __fastcall GetKeyID(const AnsiString &keystr)
{
   AnsiString longid = GetTok(lkfKeyID, keystr, ':');
   return "0x"+longid.SubString(longid.Length()-7, 8);
}
//---------------------------------------------------------------------------

AnsiString __fastcall GetUserID(const AnsiString &keystr)
{
   return GetTok(lkfKeyUID, keystr, ':');
}
//---------------------------------------------------------------------------

bool __fastcall MatchKeyIDs(AnsiString ID1, AnsiString ID2)
{
   if (ID1.LowerCase().Pos("0x") == 1)
	  ID1 = ID1.SubString(3, ID1.Length());
   if (ID2.LowerCase().Pos("0x") == 1)
	  ID2 = ID2.SubString(3, ID2.Length());
   return ID1.LowerCase() == ID2.LowerCase();
}
//---------------------------------------------------------------------------

void __fastcall TPgpDlg::CheckReportBlendAttack(const AnsiString &subno,
   const AnsiString &keyid)
{
   AnsiString msgkey, nymkey, msgkeystr, nymkeystr;
   msgkeystr = GPGGetPublicKeyStr(keyid);
   msgkey = GetKeyID(msgkeystr);
   nymkeystr = GPGGetPublicKeyStr(DB->GetSubjectItem(subno, "keyid", ""));
   nymkey = GetKeyID(nymkeystr);
   if (!MatchKeyIDs(msgkey, nymkey))
   {
	  AnsiString msgrecip, nymrecip;
	  msgrecip = msgkey + " " + GetUserID(msgkeystr);
	  nymrecip = nymkey + " " + GetUserID(nymkeystr);
	  ShowPossibleBlendingAttack(this, msgrecip, nymrecip);
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall GetNymType(const AnsiString &nym)
{
   AnsiString type, domain = GetTok(1, nym, '@');
   TStringList *server = new TStringList();

   DB->LoadNymserver(domain, server);
   assert(server->Count);
   type = server->Values["type"];

   delete server;
   return type;
}
//---------------------------------------------------------------------------

AnsiString __fastcall AsymmetricLayerStr()
{
   return "This is the public key encrypted, nymserver signed layer "
	  "of your NYM message.";
}
//---------------------------------------------------------------------------

AnsiString __fastcall SymmetricLayerStr(int layer)
{
   return "This is symmetric encryption layer "+IntToStr(layer)+" "
	  "of your incoming NYM message.";
}
//---------------------------------------------------------------------------

bool __fastcall TPgpDlg::DecryptSymmetric()
{
   assert(lines->IndexOf(BeginPGPMessage) >= 0);
   // check and decrypt symmetric encryption layer(s)
   ShowStatus("...decrypting...");
   AnsiString password;
   TStringList *keys = new TStringList();
   TStringList *words = new TStringList();
   TStringList *feedback = new TStringList();
   // parse comma-separated password list
   int i = 0; while ((password = GetTok(i, passphrase, ',')) != "")
   {
	  words->Add(password);
	  i++;
   }

   // decrypt encrypt-key layers 1 password at a time
   i = 0; while (i < words->Count)
   {
	  //GPGGetEncryptionKeys4(lines, keys);
	  /*if (!keys->Text.LowerCase().Pos("passphrase"))
	  {
		 MsgBox(this,
			"Your AAM Subject Line profile shows this message has "+IntToStr(words->Count)+" layers   \n"
			"of symmetric encryption, but I'm on layer "+IntToStr(i+1)+" and the message is   \n"
			"encrypted by a PGP public key. You should straighted out your   \n"
			"configuration. I'm skipping further symmetric decryption.",
			 MT_INFO, MB_OK);
		 delete feedback;
		 delete words;
		 delete keys;
		 return true;
	  }*/
	  password = words->Strings[i];

	  if (!GPGDecryptMessage4(this, lines, password, true, feedback))
	  {
		 if (feedback->Text.Pos("BAD_PASSPHRASE") > 0)
		 {
			AnsiString msg = "Bad passphrase!   ";
			if (words->Count > 1)
			   msg = "Bad passphrase at position "+IntToStr(i+1)+".   ";
			MsgBox(this, msg, MT_ERROR, MB_OK);
		 }
		 else
			ShowGPGError(this, feedback);
		 delete feedback;
		 delete words;
		 delete keys;
		 return false;
	  }
	  i++;
   }
   delete feedback;
   delete words;
   delete keys;
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TPgpDlg::DecryptMessage(TStrings *Message)
{
   if (GPGIsPGPSignedMessage(Message))
   {
	  ShowStatus("...verifying...");
	  Snooze(100);
	  return GPGVerifyClearSigned4(this, Message);
   }

   AnsiString idtext8;
   int tries = 0, maxtries = 3;
   TStringList *keys = new TStringList();

   tries = 0;
   bool ok = true;
   ShowStatus("getting encryption keys");
   if (GPGGetEncryptionKeys4(Message, keys))
   {
	  idtext8 = keys->Strings[0];
	  for (int i = 0; i < keys->Count; i++)
	  {
		 AnsiString key = keys->Strings[i];
		 if (!key.Pos("00000000") && !key.LowerCase().Pos("passphrase"))
			keys->Strings[i] = key + " " + GPGGetUserID(key);
	  }
   }
   else
   {
	  MsgBox(this,
		 "Sorry, I failed to get the encryption key(s) of this message.   ",
		  MT_ERROR, MB_OK);
	  delete keys;
	  return false;
   }

   idtext8 = GetTok(0, keys->Strings[0], ' ');;
   PGPCache->Get(idtext8, passphrase);
   while (ok && tries < maxtries)
   {
	  tries++;
	  if (passphrase.IsEmpty())
	  {
		 passphrase = GetDecryptionPassphrase(this, keys);
		 if (passphrase.IsEmpty())
		 {
			delete keys;
			return false;
		 }
	  }

	  ShowStatus("...decrypting...");
	  TStringList *feedback = new TStringList();
	  if (!GPGDecryptMessage4(this, Message, passphrase, true, feedback))
	  {
		 Snooze(500);
		 if (feedback->Text.Pos("BAD_PASSPHRASE") && feedback->Text.Pos("DECRYPTION_FAILED"))
		 {
			MsgBox(this, "   Bad passphrase!   ", MT_INFO, MB_OK);
			ZeroString(passphrase);
			continue;
		 }
		 else
		 {
			ShowGPGError(this, feedback);
			delete feedback;
			delete keys;
			return false;
		 }
	  }
	  delete feedback;
	  break;
   }

   PGPCache->Add(idtext8, passphrase);
   ShowStatus("");
   delete keys;

   return true;
}
//---------------------------------------------------------------------------

/*void __fastcall TPgpDlg::VerifyMessageUserSignature(TStrings *Lines)
{
   TStringList *headers = new TStringList();
   while (Lines->Count && Lines->Strings[0].Pos("***") != 1)
   {
	  headers->Add(Lines->Strings[0]);
	  Lines->Delete(0);
   }
   while (Lines->Count && Lines->Strings[0].Pos("***") == 1)
   {
	  headers->Add(Lines->Strings[0]);
	  Lines->Delete(0);
   }
   while (Lines->Count && Lines->Strings[0].Trim().IsEmpty())
   {
	  headers->Add(Lines->Strings[0]);
	  Lines->Delete(0);
   }

   GPGVerifyClearSigned4(this, Lines);

   for (int i = 0; i < headers->Count; i++)
	  Lines->Insert(i, headers->Strings[i]);
   delete headers;
}*/
//---------------------------------------------------------------------------

void __fastcall GetNymserverSig(TStrings *Message, TStrings *Sig)
{
   assert(Message != NULL);
   assert(Sig != NULL);
   Sig->Clear();
   int x = Message->IndexOf(PGPSignatureVerification);
   if (x >= 0)
   {
	  while (x < Message->Count)
	  {
		 AnsiString line = Message->Strings[x];
		 if (line.Pos("***") != 1)
			break;
		 Sig->Add(line);
		 x++;
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall StripSymmetricDecryptionMarkers(TStrings *lines)
{
   TMsgManager *msg = new TMsgManager(lines);
   int x = msg->EOH() + 1;
   assert(lines->IndexOf(BeginPGPDecryptedMessage) >= 0);
   while (lines->Count && (lines->Strings[x].Trim().IsEmpty() || lines->Strings[x] == BeginPGPDecryptedMessage) ||
	  lines->Strings[x] == "**")
		 lines->Delete(x);
   assert(lines->IndexOf(EndPGPDecryptedMessage) >= 0);
   while (lines->Count && (lines->Strings[lines->Count-1].Trim().IsEmpty() ||
	  lines->Strings[lines->Count-1] == EndPGPDecryptedMessage))
		 lines->Delete(lines->Count - 1);
   delete msg;
}
//---------------------------------------------------------------------------

bool __fastcall TPgpDlg::DecryptNymMessage(TStrings *Lines)
{
//look(Lines);
   assert(GPGIsPGPMessage(Lines));
   AnsiString nymhdr = Lines->Strings[0];
   lines->Delete(0);

   if (GPGIsPGPSignedMessage(Lines))
   {
	  return GPGVerifyClearSigned4(this, Lines);
   }

   if (GPGHasPGPSignatureVerification(Lines))
   {
	  return DecryptMessage(Lines);
   }

   TStringList *keys = new TStringList();
   ShowStatus("getting encryption keys");
   GPGGetEncryptionKeys4(Lines, keys);
   if (!keys->Count)
   {
	  MsgBox(this,
		 "Sorry, GPG failed to get the encryption key(s) for this message.   ",
		  MT_ERROR, MB_OK);
	  delete keys;
	  return false;
   }

   TMsgManager *msg = new TMsgManager(Lines);
   AnsiString password, kphrase;
   assert(subno != "");

   bool symmetric = keys->Strings[0].LowerCase().Pos("passphrase") > 0;
   if (symmetric && DB->GetSubjectItem(subno, "ekey", "0") == "1")
   {
	  passphrase = DB->GetSubjectItem(subno, "kphrase", "");
	  if (passphrase.Pos(",") > 0)
	  {
		 if (DSABox(this, "commas",
			"I see your Encrypt-Key password contains a comma. In case you didn't "
			"know, commas are reserved for separating multiple passwords in "
			"the Encrypt-Key field. If you have a single password that contains a "
			"comma, the decryption will fail.", "Encrypt-Key Comma Usage",
			 MB_OKCANCEL) == IDCANCEL)
		 {
			Hide();
			delete keys;
			delete msg;
			return false;
		 }
	  }

	  if (!DecryptSymmetric())
	  {
		 Hide();
		 delete keys;
		 delete msg;
		 return false;
	  }
	  if (!GPGIsPGPMessage(Lines))
	  {
		 MsgBox(this,
			"This doesn't look like a nym message. I've decrypted the   \n"
			"symmetric layer(s) and there is not a PGP encrypted   \n"
			"message inside!",
			 MT_INFO, MB_OK);
		 Hide();
		 delete keys;
		 delete msg;
		 return false;
	  }

	  StripSymmetricDecryptionMarkers(Lines);
	  ZeroString(passphrase);
	  keys->Clear();
   }

   delete msg;
//look(Lines);

   int tries = 0, maxtries = 5;
   ShowStatus("getting encryption keys");
   while (!keys->Count && tries < maxtries)
   {
	  if (!GPGGetEncryptionKeys4(Lines, keys))
	  {
		 delete keys;
		 return false;
	  }
	  tries++;
   }
   if (!keys->Count)
   {
	  MsgBox(this,
		 "Problem! I've attempted to get the encryption keys    \n"
		 "from this message 5 times, with no luck! I don't know   \n"
		 "why. Sorry, I can't decrypt the message. Please give   \n"
		 "it another try or 2",
		  MT_ERROR, MB_OK);
	  Hide();
	  delete keys;
	  return false;
   }
   Snooze(10);

   TIniFile *ini = new TIniFile(AppIni());
   bool IGNORECKT = ini->ReadBool("settings", "IGNORECKT", false);
   delete ini;

   bool isthrowkey = keys->Strings[0].Pos("00000000") > 0;

   if (isthrowkey)
	  keys->Strings[0] = DB->GetSubjectItem(subno, "keyid", "0x00000000");
   else
   {
	  if (keys->Strings[0].Pos("passphrase"))
	  {
		 if (!IGNORECKT && MsgBox(this,
			"PROBLEM!  I decrypted the symmetrically encrypted layer(s) of    \n"
			"your message--if there were any. However, I have found another   \n"
			"layer that GPG says is symmetric also. This could happen for a   \n"
			"couple reasons.  \n\n"
			"1)  You failed to list ALL the symmetric Encrypt-Key passphrases   \n"
			"in this nym's AAM Subject Line profile. If so, cancel this dialog  \n"
			"and fix your profile.\n\n"
			"2)  You are using a key generated by PGP-6.5.8CKT. It's the 'CKT'   \n"
			"release that is important. Only the 'CKT' version. It produces   \n"
			"keys that are not completely compatible with GPG. Even though   \n"
			"GPG says this is encrypted to a passphrase (rather than a pgp   \n"
			"key) it will decrypt for you. Just use your private key passphrase.   \n"
			"It's likely that you won't find a pgp signature from the nymserver        \n"
			"either. If this is your problem, it's time for you to consider   \n"
			"joining the 21st century an start using GPG!",
		    MT_INFO, MB_OKCANCEL) == IDCANCEL)
		 {
			return false;
		 }
	  }
	  else
		 CheckReportBlendAttack(subno, keys->Strings[0]);
   }
   tries = 0;
   AnsiString idtext8;
   TStringList *passphrases = new TStringList();
   idtext8 = keys->Strings[0];
   PGPCache->Get(idtext8, passphrase);
   while (true)
   {
	  ok = true;
	  //if (passphrases->Count)
	  //	  passphrase = passphrases->Strings[0];
	  if (passphrase.IsEmpty())
	  {
		 for (int i = 0; i < keys->Count; i++)
		 {
			if (keys->Strings[i] != "0x00000000" && keys->Strings[i] != "1 passphrase")
			{
			   keys->Strings[i] = keys->Strings[i] + " " + GPGGetUserID(keys->Strings[i]);
			}
		 }
		 passphrase = GetNymDecryptionPassphrase(this, keys, isthrowkey);
	  }
	  if (passphrase.IsEmpty())
	  {
		 Hide();
		 delete keys;
		 return false;
	  }

	  ShowStatus("...decrypting...");
	  Snooze(10);
	  bool quiet = passphrases->Count > 0;
	  TStringList *feedback = new TStringList();

	  if (!GPGDecryptMessage4(this, Lines, passphrase, true, feedback))
	  {
		 if (!quiet)
		 {
			Snooze(500);
			MessageBeep(MB_ICONHAND);
			if (feedback->Text.Pos("BAD_PASSPHRASE") && feedback->Text.Pos("DECRYPTION_FAILED"))
			{
			   MsgBox(this, "   Bad passphrase!   ", MT_INFO, MB_OK);
			}
			else
			{
			   ShowGPGError(this, feedback);
			}
		 }

		 ok = false;
		 //if (passphrases->Count > 0)
		 //	passphrases->Delete(0);
		 //else
		 //{
			tries++;
		 //}
		 if (tries < maxtries)
		 {
			ZeroString(passphrase);
			continue;
		 }
		 Hide();
		 delete feedback;
		 delete keys;
		 return false;
	  }

	  if (isthrowkey)
	  {
		 AnsiString keyid;
		 for (int i = feedback->Count - 1; i > 0; i--)
		 {
			AnsiString line = feedback->Strings[i];
			if (line.Pos("[GNUPG:] NEED_PASSPHRASE") > 0)
			{
			   keyid = GetTok(2, line, ' ');
			   break;
			}
		 }
		 CheckReportBlendAttack(subno, keyid);
	  }

	  PGPCache->Add(idtext8, passphrase);
	  ShowStatus("");

	  TStringList *sig = new TStringList();
	  GetNymserverSig(Lines, sig);
	  if (!IGNORECKT)
		 CheckShowNymserverSignatureFailed(this, sig, nymhdr);

	  delete sig;
	  delete feedback;
	  break;
   }
   return ok;
}
//---------------------------------------------------------------------------

/*bool __fastcall TPgpDlg::DecryptNymMessage(TStrings *Lines)
{
   assert(GPGIsPGPMessage(Lines));
   AnsiString nymhdr = Lines->Strings[0];
   lines->Delete(0);

   if (GPGIsPGPSignedMessage(Lines))
   {
	  return GPGVerifyClearSigned4(this, Lines);
   }

   if (GPGHasPGPSignatureVerification(Lines))
   {
	  return DecryptMessage(Lines);
   }

   TStringList *keys = new TStringList();
   ShowStatus("getting encryption keys");
   Snooze(200);
   GPGGetEncryptionKeys4(Lines, keys);
   assert(keys->Count);

   TMsgManager *msg = new TMsgManager(Lines);
   AnsiString password, kphrase;
   assert(subno != "");

   bool symmetric = keys->Strings[0].LowerCase().Pos("passphrase") > 0;
   if (symmetric && DB->GetSubjectItem(subno, "ekey", "0") == "1")
   {
	  passphrase = DB->GetSubjectItem(subno, "kphrase", "");
	  if (passphrase.Pos(",") > 0)
	  {
		 if (DSABox(this, "commas",
			"I see your Encrypt-Key password contains a comma. In case you didn't "
			"know, commas are reserved for separating multiple passwords in "
			"the Encrypt-Key field. If you have a single password that contains a "
			"comma, the decryption will fail.", "Encrypt-Key Comma Usage",
			 MB_OKCANCEL) == IDCANCEL)
		 {
			Hide();
			delete keys;
			delete msg;
			return false;
		 }
	  }

	  if (!DecryptSymmetric())
	  {
		 Hide();
		 delete keys;
		 delete msg;
		 return false;
	  }

	  if (!GPGIsPGPMessage(Lines))
	  {
		 MsgBox(this,
			"This doesn't look like a nym message. I've decrypted the   \n"
			"symmetric layer(s) and there is not a PGP encrypted   \n"
			"message inside!",
			 MT_INFO, MB_OK);
		 Hide();
		 delete keys;
		 delete msg;
		 return false;
	  }

	  StripSymmetricDecryptionMarkers(Lines);
	  ZeroString(passphrase);
	  keys->Clear();
   }

   delete msg;

   int tries = 0, maxtries = 5;
   ShowStatus("getting encryption keys");
   while (!keys->Count && tries < maxtries)
   {
	  if (!GPGGetEncryptionKeys4(Lines, keys))
	  {
		 delete keys;
		 return false;
	  }
	  tries++;
   }
   if (!keys->Count)
   {
	  MsgBox(this,
		 "Problem! I've attempted to get the encryption keys    \n"
		 "from this message 5 times, with no luck! I don't know   \n"
		 "why. Sorry, I can't decrypt the message. Please give   \n"
		 "it another try or 2",
		  MT_ERROR, MB_OK);
	  Hide();
	  delete keys;
	  return false;
   }
   Snooze(10);

   bool isthrowkey = keys->Strings[0].Pos("00000000") > 0;
   if (isthrowkey)
	  keys->Strings[0] = DB->GetSubjectItem(subno, "keyid", "0x00000000");
   else
	  CheckReportBlendAttack(subno, keys->Strings[0]);

   tries = 0;
   TStringList *feedback = new TStringList();
   AnsiString idtext8 = keys->Strings[0];
   PGPCache->Get(idtext8, passphrase);
   while (true)
   {
	  ok = true;
	  if (passphrase.IsEmpty())
	  {
		 //for (int i = 0; i < keys->Count; i++)
		 //{
		 //	if (keys->Strings[i] != "0x00000000")
		 //	   keys->Strings[i] = keys->Strings[i] + " " + GPGGetUserID(keys->Strings[i]);
		 //}
		 passphrase = GetNymDecryptionPassphrase(this, keys, isthrowkey);
	  }
	  if (passphrase.IsEmpty())
	  {
		 Hide();
		 delete keys;
		 return false;
	  }

	  ShowStatus("...decrypting...");
	  Snooze(10);
	  TStringList *feedback = new TStringList();

	  if (!GPGDecryptMessage4(this, Lines, passphrase, true, feedback))
	  {
		 Snooze(500);
		 MessageBeep(MB_ICONHAND);
		 if (feedback->Text.Pos("BAD_PASSPHRASE") && feedback->Text.Pos("DECRYPTION_FAILED"))
		 {
			MsgBox(this, "   Bad passphrase!   ", MT_INFO, MB_OK);
		 }
		 else
		 {
			ShowGPGError(this, feedback);
		 }
	  }
	  else
	  {
		 sm(passphrase);
		 break;
	  }

	  ok = false;
	  tries++;
	  if (tries < maxtries)
	  {
		 ZeroString(passphrase);
		 continue;
	  }
	  Hide();
	  delete feedback;
	  delete keys;
	  return false;
   }

   if (isthrowkey)
   {
	  AnsiString keyid;
	  for (int i = feedback->Count - 1; i > 0; i--)
	  {
		 AnsiString line = feedback->Strings[i];
		 if (line.Pos("[GNUPG:] NEED_PASSPHRASE") > 0)
		 {
			keyid = GetTok(2, line, ' ');
			break;
		 }
	  }
	  CheckReportBlendAttack(subno, keyid);
   }

//sm(idtext8+"  "+passphrase);
   PGPCache->Add(idtext8, passphrase);
   ShowStatus("");

   TStringList *sig = new TStringList();
   GetNymserverSig(Lines, sig);

   CheckShowNymserverSignatureFailed(this, sig, nymhdr);

   delete sig;
   delete feedback;

   return ok;
}*/
//---------------------------------------------------------------------------

int  __fastcall TPgpDlg::OwnerPerform(UINT Msg, WPARAM wParam, LPARAM lParam)
{
   return ((TForm*)Owner)->Perform(Msg, wParam, lParam);
}
//---------------------------------------------------------------------------

TForm* __fastcall TPgpDlg::OwnerForm()
{
   return (TForm*)Owner;
}
//---------------------------------------------------------------------------


