/*
*  proxymandlgs.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "proxymandlgs.h"
#include "proxydlgs.h"
#include "utility.h"
#include "global.h"
#include "dbs.h"
#include "qsadbs.h"
#include "help.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TProxyManDlg *ProxyManDlg;
//---------------------------------------------------------------------------

bool __fastcall ManageProxies(TForm *Owner)
{
   TProxyManDlg *proxyman = new TProxyManDlg(Owner);
   proxyman->ShowModal();
   bool ok = proxyman->modified;
   delete proxyman;
   return ok;

}
//---------------------------------------------------------------------------

__fastcall TProxyManDlg::TProxyManDlg(TComponent* Owner)
		: TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "proxymandlg", ListView, ini);
   orgwin = IntToStr(Width)+IntToStr(Height) +
	  IntToStr(ListView->Columns->Items[0]->Width) +
	  IntToStr(ListView->Columns->Items[1]->Width) +
	  IntToStr(ListView->Columns->Items[2]->Width);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(ListView->Font, ini->ReadString("settings", "windowfont",
	  OptsDefWindowFont));
   defnntp = ini->ReadString("settings", "nntpproxy", "none");
   delete ini;

   TStringList *proxies = new TStringList();
   DB->GetText("proxies", proxies);
   LoadSelectorBox(proxies);
   SetProxies(proxies);
   delete proxies;
   modified = false; // LoadSelectorBox sets modified true
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::FormCloseQuery(TObject *Sender,
	  bool &CanClose)
{
   if (modified)
   {
	  if (ModalResult == mrCancel)
	  {
		 switch (MsgBox(this,
			"Your proxy info has been modified. Save your changes?   \n",
			 MT_CONFIRM, MB_YESNOCANCEL))
		 {
			case IDCANCEL: CanClose = false;
			case IDNO: return;
		 }
	  }

	  TStringList *proxies = new TStringList();
	  GetProxies(proxies);
	  DB->PutText("proxies", proxies);
	  delete proxies;
   }
}
//---------------------------------------------------------------------------

__fastcall TProxyManDlg::~TProxyManDlg()
{
   if (IntToStr(Width)+IntToStr(Height)+
	   IntToStr(ListView->Column[0]->Width)+
	   IntToStr(ListView->Column[1]->Width)+
	   IntToStr(ListView->Column[2]->Width) != orgwin)
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  SaveWindowPosition(this, "proxymandlg", ListView, ini);
	  delete ini;
   }

   if (defnntpchanged)
	  Application->MainForm->Perform(WM_INICHANGE, 0, 0);
}
//---------------------------------------------------------------------------

/*bool __fastcall TProxyManDlg::LoadProxyBox(TStrings *proxies, AnsiString &defnntp)
{
   TIniFile *ini = new TIniFile(AppIni());
   defnntp = ini->ReadString("settings", "nntpproxy", "none");
   DB->GetText("proxies", proxies);
look(proxies);
   ProxyBox->Items->Clear();
   ProxyBox->Sorted = true;
   for (int i = 0; i < proxies->Count; i++)
	  ProxyBox->Items->Add(proxies->Names[i]);
   ProxyBox->Sorted = false;
   ProxyBox->Items->Insert(0, "none");

   ProxyBox->ItemIndex = ProxyBox->Items->IndexOf(defnntp);
   if (ProxyBox->ItemIndex < 0)
   {
	  MsgBox(Screen->ActiveForm,
		 "The NNTP proxy you've been using, " + defnntp + ", is missing! You   \n"
		 "don't currently have a proxy assigned!",
		  MT_WARN, MB_OK);
	  defnntp = "none";
	  ini->WriteString("settings", "smtpproxy", defnntp);
	  ProxyBox->ItemIndex = 0;
   }
   delete ini;
   return ProxyBox->Items->Count > 1;
}*/
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::LoadSelectorBox(TStrings *proxies)
{
   TStringList *proxy = new TStringList();

   NNTPBox->Items->Clear();
   NNTPBox->Sorted = true;
   for (int i = 0; i < proxies->Count; i++)
   {
	  NNTPBox->Items->Add(proxies->Names[i]);
   }
   NNTPBox->Sorted = false;

   NNTPBox->Items->Insert(0, "none");
   NNTPBox->ItemIndex = NNTPBox->Items->IndexOf(defnntp);
   CheckReportDefaultProxyMissing();

   ProxyBoxSelect(NNTPBox);

   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::CheckReportDefaultProxyMissing()
{
   if (NNTPBox->ItemIndex < 0)
   {
	  MsgBox(this,
		 "The NNTP proxy you've been using, " + defnntp + ", is missing! You  \n"
		 "don't currently have a proxy assigned.",
		 MT_WARN, MB_OK);
	  NNTPBox->ItemIndex = 0;
	  defnntp = "none";
	  TIniFile *ini = new TIniFile(AppIni());
	  ini->WriteString("settings", "nntpproxy", defnntp);
	  delete ini;
   }
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::SetProxies(TStrings *proxies)
{
   TStringList *proxy = new TStringList();

   ListView->Items->Clear();
   ListView->Items->BeginUpdate();
   for (int i = 0; i < proxies->Count; i++)
   {
	  proxy->CommaText = proxies->Values[proxies->Names[i]];
	  TListItem *item = ListView->Items->Add();
	  SetProxy(item, proxy);
   }
   ListView->Items->EndUpdate();

   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::GetProxy(TListItem *item, TStrings *proxy)
{
   assert(item != NULL);
   assert(proxy != NULL);
   proxy->Clear();
   proxy->Values["proxy"] = item->Caption;
   proxy->Values["host"] = item->SubItems->Strings[0];
   proxy->Values["port"] = item->SubItems->Strings[1];
   proxy->Values["socks"] = item->SubItems->Strings[2];
   proxy->Values["tor"] = item->SubItems->Strings[3] == "*" ? "1" : "0";
   proxy->Values["auth"] = item->SubItems->Strings[4];
   proxy->Values["userid"] = item->SubItems->Strings[5];
   proxy->Values["password"] = item->SubItems->Strings[6];
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::SetProxy(TListItem *item, TStrings *proxy)
{
   AnsiString proto;
   while (item->SubItems->Count < 7)
	  item->SubItems->Add("");

   item->Caption = proxy->Values["proxy"];
   item->SubItems->Strings[0] = proxy->Values["host"];
   item->SubItems->Strings[1] = proxy->Values["port"];
   item->SubItems->Strings[2] = proxy->Values["socks"];
   item->SubItems->Strings[3] = proxy->Values["tor"] == "1" ? "*" : "";
   item->SubItems->Strings[4] = proxy->Values["auth"];
   item->SubItems->Strings[5] = proxy->Values["userid"];
   item->SubItems->Strings[6] = proxy->Values["password"];
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::GetProxies(TStrings *proxies)
{
   TStringList *proxy = new TStringList();

   proxies->Clear();
   for (int i = 0; i < ListView->Items->Count; i++)
   {
	  TListItem *item = ListView->Items->Item[i];
	  GetProxy(item, proxy);
	  proxies->Values[item->Caption] = proxy->CommaText;
   }
   ListView->Items->EndUpdate();

   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::PopupMenu1Popup(TObject *Sender)
{
   Edit1->Enabled = ListView->Selected;
   Delete1->Enabled = ListView->Selected;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Resizedialog1Click(TObject *Sender)
{
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Locksize1Click(TObject *Sender)
{
   BorderStyle = bsDialog;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   CheckReportDefaultProxyMissing();
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::NNTPBoxChange(TObject *Sender)
{
   assert(NNTPBox->Text != "");
   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", "nntpproxy", NNTPBox->Text);
   defnntp = NNTPBox->Text;
   defnntpchanged = true;
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Add1Click(TObject *Sender)
{
   TStringList *proxy = new TStringList();
   TStringList *proxies = new TStringList();

   GetProxies(proxies);
   if (NewProxy(this, proxy, proxies))
   {
      TListItem *item = ListView->Items->Add();
	  SetProxy(item, proxy);
	  LoadSelectorBox(proxies);
	  modified = true;
   }

   delete proxies;
   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Edit1Click(TObject *Sender)
{
   assert(ListView->Selected != NULL);
   TStringList *proxy = new TStringList();
   TStringList *proxies = new TStringList();

   GetProxies(proxies);
   GetProxy(ListView->Selected, proxy);
   AnsiString newname, oldname = proxy->Values["proxy"];
   if (EditProxy(this, proxy, proxies))
   {
	  newname = proxy->Values["proxy"];
	  if (newname != oldname)
	  {
		 if (NNTPBox->Text == oldname)
		 {
			NNTPBox->Items->Delete(NNTPBox->ItemIndex);
			NNTPBox->ItemIndex = NNTPBox->Items->Add(newname);
			NNTPBoxChange(NULL);
		 }
	  }
	  SetProxy(ListView->Selected, proxy);
	  modified = true;
   }
   delete proxies;
   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Delete1Click(TObject *Sender)
{
   assert(ListView->Selected != NULL);
   AnsiString name = ListView->Selected->Caption;
   if (MsgBox(this,
	  "Delete the proxy, " + name + "?  \n",
	   MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
	  ListView->Selected->Delete();
	  DeleteBtn->Enabled = false;
	  EditBtn->Enabled = false;
	  modified = true;
	  int x = NNTPBox->Items->IndexOf(name);
	  if (x >= 0)
	  {
		 if (NNTPBox->Text == name)
		 {
			NNTPBox->ItemIndex = 0;
			TIniFile *ini = new TIniFile(AppIni());
			ini->WriteString("settings", "nntpproxy", "none");
			delete ini; Snooze(200);
			MsgBox(this,
			  "That was your default NNTP proxy. Now, there's none assigned.  ",
			   MT_WARN, MB_OK);
			defnntp = "none";
		 }
		 NNTPBox->Items->Delete(x);
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::ListViewClick(TObject *Sender)
{
   EditBtn->Enabled = ListView->Selected;
   DeleteBtn->Enabled = EditBtn->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::ProxyBoxSelect(TObject *Sender)
{
   TComboBox *ProxyBox = dynamic_cast<TComboBox*>(Sender);
   if (!ProxyBox) return;

   if (ProxyBox->Text == "none")
	 ProxyBox->Font->Style = TFontStyles() << fsBold;
   else
	 ProxyBox->Font->Style = TFontStyles();

   AnsiString theproxy;
   if (ProxyBox == NNTPBox)
   {
	  defnntp = ProxyBox->Text;
	  theproxy = "nntpproxy";
   }

   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", theproxy, ProxyBox->Text);
   delete ini;

   modified = true;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::ProxyBoxDropDown(TObject *Sender)
{
   TComboBox *ProxyBox = dynamic_cast<TComboBox*>(Sender);
   if (!ProxyBox) return;

   ProxyBox->Font->Style = TFontStyles();
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::ProxyBoxCloseUp(TObject *Sender)
{
   TComboBox *ProxyBox = dynamic_cast<TComboBox*>(Sender);
   if (!ProxyBox) return;

   if (ProxyBox->Text == "none")
	 ProxyBox->Font->Style = TFontStyles() << fsBold;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::HelpBtnClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/proxyman.htm>proxyhelp");
}
//---------------------------------------------------------------------------

