/*
*  secdbs.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <math.h>
#include <stdio.h>

#include "secdbs.h"
#include "gpg4.h"
#include "msgmans.h"
#include "utility.h"
#include "dbs.h"
#include "eraser.h"
#include "pgpdlgs.h"
//#include "mixmaster.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDBSecModeDlg *DBSecModeDlg;
TSecureDB *SecureDB;
//---------------------------------------------------------------------------

//int __fastcall rnd_bytes(byte *buffer, int length)
//{
//   sm("Needs rnd_bytes!");
//   return false;
//}

//---------------------------------------------------------------------------
// flag functions
//---------------------------------------------------------------------------
// WORD FLAGS
void __fastcall SetItemFlag(AnsiString &key, WORD flag)
{
   *(WORD*)&key[wflags] |= flag;
}
//---------------------------------------------------------------------------

void __fastcall UnSetItemFlag(AnsiString &key, WORD flag)
{
   *(WORD*)&key[wflags] &= (WORD)~flag;
}
//---------------------------------------------------------------------------

bool __fastcall ItemFlagIsSet(AnsiString key, WORD flag)
{
   return (*(WORD*)&key[wflags] & flag) == flag;
}

//---------------------------------------------------------------------------
// GPG Functions
//---------------------------------------------------------------------------

bool __fastcall GPGDecryptDatabase(TSecureDB *DB, AnsiString &Passphrase)
{
   if (!checkgpgavailable())
	  return false;

   AnsiString cmdline = getcommandline(GPGdecryptdb, GPGDecryptDBCmd),
	  filename = AppDB();

   // get unencrypted file size from end of encrypted file
   __int64 dbSize;
   int size, size64 = sizeof(__int64);
   TFileStream *strm = new TFileStream(filename, fmOpenRead | fmShareExclusive);
   strm->Seek(-size64, soFromEnd);
   strm->Read(&dbSize, size64);
   size = dbSize > 2147483648 ? 2147483648 : (int)dbSize;
   DB->memstream->SetSize(size);
   delete strm;

   setfield("%filename%", GetShortName(filename), cmdline);

   TGPGConsole4 *con = new TGPGConsole4(Screen->ActiveForm);
   bool ok = con->Run(cmdline, Passphrase.c_str(), NULL, DB->stream);
   if (ok)
   {
	  Snooze(200);
	  if (con->status->Text.Pos("DECRYPTION_FAILED"))
	  {
		 if (con->error->Text.Pos("bad key"))
		 {
			MsgBox(Screen->ActiveForm,
			  "Bad passphrase!    ",
			   MT_ERROR, MB_OK);
		 }
		 else
		 {
			ShowGPGError(Screen->ActiveForm, con->error, con->status);
		 }
		 ok = false;
	  }
   }
   delete con;

   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall GPGEncryptDatabase(TSecureDB *DB, AnsiString &Passphrase)
{
   if (!checkgpgavailable())
	  return false;

   AnsiString cmdline = getcommandline(GPGencryptdb, GPGEncryptDBCmd),
	  tmpname = AppDBTmp();
   if (FileExists(tmpname))
	  DeleteFile(tmpname);

   setfield("%filename%", tmpname, cmdline);

   TMemoryStream *output = new TMemoryStream();
   TGPGConsole4 *con = new TGPGConsole4(Screen->ActiveForm);
   bool ok = con->Run(cmdline, Passphrase.c_str(), DB->stream, output);  // RTC
   ZeroString(Passphrase);
   if (ok)
   {
	  // append the size of the unencrypted database to end of encrypted database
	  __int64 dbSize = DB->stream->Size;
	  TFileStream *strm = new TFileStream(tmpname, fmOpenWrite | fmShareExclusive);
	  strm->Seek(0, soFromEnd);
	  strm->Write(&dbSize, sizeof(__int64));
	  delete strm;

	  // delete original db and rename tmpname to AppDB()
	  if (DB->filename != AppDBTmp())   // only wipe/rename if DB isn't AppDBTmp()
	  {                                 // (when packing database)
		 if (FileExists(AppDB()))
		 {
			if (!GPGIsFileEncrypted(AppDB())) // || wipeencryptedDB()) // RTC
			   eraserWipeFile(AppDB(), false);
			else
			   DeleteFile(AppDB());
		 }
		 RenameFile(tmpname, AppDB());
		 DB->modified = false;
	  }
   }
   else
   {
	  Sleep(200);
	  ShowGPGError(Screen->ActiveForm, con->error, con->status);
   }
   delete con;
   delete output;

   return ok;
}

//---------------------------------------------------------------------------
// SecureDB
//---------------------------------------------------------------------------

TSecureDB* __fastcall OpenSecureDB(const AnsiString &FileName)
{
   assert(FileName != "");
   TSecureDB *DB = new TSecureDB(FileName);
   if (DB->error)
   {
	  delete DB;
	  return NULL;
   }
   return DB;
}
//---------------------------------------------------------------------------

bool __fastcall CloseSecureDB(TSecureDB *DB)
{
   return CloseDB(DB);
}

//---------------------------------------------------------------------------
// TSecureDB
//---------------------------------------------------------------------------

__fastcall TSecureDB::TSecureDB(const AnsiString &FileName, bool InMemory)
   : TDB(FileName, InMemory)
{
// unused
}
//---------------------------------------------------------------------------

__fastcall TSecureDB::TSecureDB(const AnsiString &FileName)
   : TDB(FileName, true)
{
}
//---------------------------------------------------------------------------

__fastcall TSecureDB::~TSecureDB()
{
   if (MasterPassword)
   {
	  ZeroMemory(MasterPassword->Memory, (SIZE_T)MasterPassword->Size);
	  delete MasterPassword;
   }
}

//---------------------------------------------------------------------------
// Secure Mode Functions
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::OpenDatabase()
{
   assert(stream == NULL);
   MasterPassword = NULL;

   AnsiString key, passphrase;
   try
   {
	  bool x = (GPGIsFileEncrypted(filename));
	  if (x)
	  {
		 int attempt = 0;
		 while (true)
		 {
			attempt++;
			if (!GetUserPassphrase(NULL, passphrase))
			{
			   error = DB_FAILEDINITERR;
			   return false;
			}

			fstream = new TMemoryStream(), inmemory = true;
			if (!GPGDecryptDatabase(this, passphrase))   // RTC
			{
			   if (attempt == 3)
			   {
				  error = DB_FAILEDINITERR;
				  delete fstream;
				  return false;
			   }
			   continue;
			}

			MasterPassword = new TMemoryStream();
			break;
		 }
	  }
	  else
	  {
		 fstream = new TMemoryStream(), inmemory = true;
		 if (!filename.IsEmpty() && FileExists(filename))
			memstream->LoadFromFile(filename);
	  }
   }
   catch(Exception &e)
   {
	  exceptmsg = e.Message;
	  error = DB_READEXC;
	  ShowDBError(error);
	  return false;
   }

   dbman = new TDBmanager(this, filename);
   if (!dbman->error)
   {
	  index[0] = new TDBindex(dbman);
	  modified = false;
   }
   else
   {
	  ShowDBError(dbman->error);
	  error = dbman->error;
   }

   CreateSessionKey();
   bool ok = true;
   if (MasterPassword)
   {
	  AnsiString key;
	  LoadSessionKey(key);
	  ok = GPGEncryptStringToStream(key, passphrase, MasterPassword);
	  ZeroString(passphrase);
	//GPGSaveMasterPassword(this, passphrase);     // RTC
   }

   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::CloseDatabase()
{
   /*if (modified && IsSecureMode())
   {
	  AnsiString passphrase, key;
	  LoadSessionKey(key);
	  GPGDecryptStreamToString(key, MasterPassword, passphrase);
	  ClearSessionKey();
	  if (!EncryptDatabase(passphrase))
	  {
		 SaveSessionKey(key);
		 ZeroString(key);
		 return false;
	  }
	  modified = false;
	  ZeroString(key);
   }*/
   return TDB::CloseDatabase();
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::EncryptDatabase(AnsiString &passphrase)
{
   ShowStatus(" encrypting... ");
   if (!GPGEncryptDatabase(this, passphrase))
   {
	  ShowStatus(" Encryption FAILED! ");
	  return false;
   }
   Sleep(200);
   ShowStatus("");
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::DecryptDatabase(AnsiString &passphrase)
{
   ShowStatus(" decrypting... ");
   if (!GPGDecryptDatabase(this, passphrase))
   {
	  ShowStatus(" Decryption FAILED! ");
	  return false;
   }
   ShowStatus("");
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TSecureDB::ShowStatus(const AnsiString &msg)
{
   try // I get an access violation when calling this after encrypting database
   {   // while closing (Secure Mode);
	  Screen->ActiveForm->Perform(WM_SHOWSTATUS, 0, (LPARAM)msg.c_str());
   }
   catch(EAccessViolation &e)
   {
   }
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::GetUserPassphrase(TForm *Owner, AnsiString &passphrase)
{
   DBSecModeDlg = new TDBSecModeDlg(Owner);
   DBSecModeDlg->SetMode(dbpDecrypt);
   DBSecModeDlg->ShowModal();
   bool ok = DBSecModeDlg->ModalResult == mrOk;
   if (ok)
   {
	  passphrase = DBSecModeDlg->Phrase->Text;
	  //DBSecModeDlg->ClearText();
   }
   delete DBSecModeDlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::IsSecureMode()
{
   return MasterPassword != NULL;
}
//---------------------------------------------------------------------------

int __fastcall TSecureDB::PermissionSecureMode(TForm *Owner)
{
   DBSecModeDlg = new TDBSecModeDlg(Owner);
   AnsiString key;
   LoadSessionKey(key);
   GPGDecryptStreamToString(key, MasterPassword, DBSecModeDlg->currentpassphrase);
   ZeroMemory(key.c_str(), key.Length());
   DBSecModeDlg->SetMode(dbpPermission);
   DBSecModeDlg->ShowModal();
   int ret = DBSecModeDlg->ModalResult;
   delete DBSecModeDlg;
   return ret;
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::ManageSecureMode(TForm *Owner)
{
   bool secure = IsSecureMode();

   if (secure)
   {
	  switch (PermissionSecureMode(Owner))
	  {
		 case IDABORT:
			try
			{
			  SecureZeroMemory(MasterPassword->Memory, (SIZE_T)MasterPassword->Size);
			}
			__finally
			{
			   delete MasterPassword;
			   MasterPassword = NULL;
			   modified = true;
			   Snooze(50);
			   MsgBox(Owner, "Secure Mode is canceled!   ", MT_INFO, MB_OK);
			   SaveSetting("securemode", "0");
			   return true;
			}
		 case IDCANCEL:
			return false;
	  }
	  Snooze(100);
   }

   bool ok = false;
   DBSecModeDlg = new TDBSecModeDlg(Owner);
   DBSecModeDlg->SetMode(secure ? dbpChange : dbpEncrypt);
   DBSecModeDlg->ShowModal();
   if (DBSecModeDlg->ModalResult == mrOk)
   {
	  AnsiString key;
	  LoadSessionKey(key);
	  MasterPassword = new TMemoryStream();
	  GPGEncryptStringToStream(key, DBSecModeDlg->currentpassphrase, MasterPassword);
	  modified = true;
	  Snooze(100);
	  MsgBox(Owner, "Secure Mode is set!    ", MT_INFO, MB_OK);
	  SaveSetting("securemode", "1");
	  ok = true;
   }
   delete DBSecModeDlg;
   return ok;
}
//---------------------------------------------------------------------------

/*bool __fastcall TSecureDB::CreateSessionKey()
{
   ClearSessionKey();

   AnsiString seskey;
   char pcData[33] = { 0 };

   // get random bytes from mixmaster
   if (rnd_bytes((byte*)pcData, sizeof(pcData) - 1) == 0)     // RTC
   {
	  seskey = Base64EncodeData(pcData, sizeof(pcData) - 1);
	  ZeroMemory(pcData, sizeof(pcData));
	  assert(seskey.Length() > 32);
   }
   else
   {
	  MsgBox(NULL,
		 "I failed to get random bytes from mixmaster for my session key.  \n"
		 "Using backup method.",
		  MT_INFO, MB_OK);

	  // guaranteed to work but lesser quality
	  //randomize(); called at app startup
	  seskey = AnsiString(time(NULL) * GetTickCount() * _threadid * random(2147483647));
	  seskey = Base64EncodeData(seskey.c_str(), seskey.Length());
   }

   bool ok = PutBuffer(TheSession+DBPS+Key, seskey.c_str(), seskey.Length());
   ZeroMemory(seskey.c_str(), seskey.Length());
   modified = false;

   return ok;
}*/
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::CreateSessionKey()
{
   ClearSessionKey();
   AnsiString seskey;
   seskey = AnsiString(time(NULL) * GetTickCount() * _threadid * random(2147483647));
   seskey = Base64EncodeData(seskey.c_str(), seskey.Length());
   //char hash[60] = {0};
   //MD5Hash(seskey, (unsigned char*)hash);
   //seskey = hash;
   TStringList *session = new TStringList();
   bool ok = GetText("session", session);
   session->Values[Key] = seskey;
   if (ok)
	  ok = PutText(TheSession, session);
   ZeroMemory(seskey.c_str(), seskey.Length());
   delete session;
   modified = false;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::LoadSessionKey(AnsiString &SesKey)
{
   TStringList *session = new TStringList();
   bool ok = GetText(TheSession, session);
   SesKey = session->Values[Key];
   delete session;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::SaveSessionKey(AnsiString &SesKey)
{
   TStringList *session = new TStringList();
   bool ok = GetText(TheSession, session);
   if (ok)
   {
	  session->Values[Key] = SesKey;
	  ok = PutText(TheSession, session);
   }
   delete session;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::ClearSessionKey()
{
   bool mod = modified;
   TStringList *session = new TStringList();
   bool ok = GetText(TheSession, session);
   session->Values[Key] = "";
   if (ok)
	  ok = PutText(TheSession, session);
   delete session;
   modified = mod;
   return ok;
}
//---------------------------------------------------------------------------

TMemoryStream* __fastcall TSecureDB::GetMemoryStream()
{
   TMemoryStream *memstrm = dynamic_cast<TMemoryStream*>(fstream);
   assert(memstrm != NULL);
   return memstrm;
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::Save()
{
   //int error = DB_OK;
   if (modified)
   {
	  if (IsSecureMode())
	  {
		 try
		 {
			AnsiString passphrase, key;
			LoadSessionKey(key);
			GPGDecryptStreamToString(key, MasterPassword, passphrase);
			if (!EncryptDatabase(passphrase))
			{
			   SaveSessionKey(key);
			   ZeroString(key);
			   return false;
			}
			modified = false;
			ZeroString(key);
		 }
		 catch(Exception &e)
		 {
			MsgBox(Screen->ActiveForm,
			  "Problem! I couldn't save the database! The error reported was:   \n\n"+
			   e.Message,
			   MT_ERROR, MB_OK);
			exceptmsg = e.Message;
			error = DB_WRITEEXC;
			return false;
		 }
	  }
	  else
	  {
		 try
		 {
		  // overwrite previous file if present
			bool exists = FileExists(filename);
			TFileStream *filestream = new TFileStream(filename,
			   (WORD)((exists ? fmOpenReadWrite : fmCreate) | fmShareExclusive));
			filestream->Seek(0, soFromBeginning);
			memstream->SaveToStream(filestream);
			if (filestream->Size > memstream->Size)
			   filestream->Size = memstream->Size;
			delete filestream;
			modified = false;
		 }
		 catch(Exception &e)
		 {
			MsgBox(Screen->ActiveForm,
			  "Problem! I couldn't save the database! The error reported was:   \n\n"+
			   e.Message,
			   MT_ERROR, MB_OK);
			exceptmsg = e.Message;
			error = DB_WRITEEXC;
			return false;
		 }
	  }
   }
   return true;
}

//---------------------------------------------------------------------------
// settings
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::LoadSettings(TStrings *settings)
{
   return GetText("settings", settings);
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::SaveSettings(TStrings *settings)
{
   return PutText("settings", settings);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TSecureDB::LoadSetting(const AnsiString &Ident, const AnsiString &Default)
{
   AnsiString val;
   TStringList *sets = new TStringList();
   bool ok = LoadSettings(sets);
   if (ok) val = sets->Values[Ident];
   if (val.IsEmpty())
	  val = Default;
   delete sets;
   return val;
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::SaveSetting(const AnsiString &Ident, const AnsiString &Value)
{
   TStringList *sets = new TStringList();
   bool ok = LoadSettings(sets);
   if (ok)
   {
	  sets->Values[Ident] = Value;
	  ok = SaveSettings(sets);
   }
   delete sets;
   return ok;
}

//---------------------------------------------------------------------------
// config
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::LoadConfig(const AnsiString &name, TStrings *config)
{
   TStringList *cfg = new TStringList();
   bool ok = GetText(Config, cfg);
   config->CommaText = cfg->Values[name];
   delete cfg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TSecureDB::SaveConfig(const AnsiString &name, TStrings *config)
{
   TStringList *cfg = new TStringList();
   bool ok = GetText(Config, cfg);
   cfg->Values[name] = config->CommaText;
   if (ok) ok = PutText(Config, cfg);
   delete cfg;
   return ok;
}

//---------------------------------------------------------------------------
// TDBPasswordDlg2
//---------------------------------------------------------------------------

__fastcall TDBSecModeDlg::TDBSecModeDlg(TComponent* Owner)
		: TForm(Owner)
{
   Position = Owner == NULL ? poScreenCenter : poOwnerFormCenter;
   if (Owner)
	  Font->Assign(((TForm*)Owner)->Font);

   minpasslen = 6;//GetAdvancedConfigItem(GPGminpasslength, GPGMinPassLength); // RTC
}
//---------------------------------------------------------------------------

__fastcall TDBSecModeDlg::~TDBSecModeDlg()
{
   WipeEditBox(Cnfirm);
   WipeEditBox(Phrase);
   ClearText();
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::SetMode(int amode)
{
   mode = amode;
   switch (mode)
   {
	  case dbpDecrypt:
		 Caption = " QuickSilver Secure Mode ";
		 Label2->Caption = "&Enter decryption passphrase :";
		 QualityBox->Hide();
		 Cnfirm->Hide();
		 Label3->Hide();
		 Bevel1->Hide();
		 //HelpBtn->Hide();
		 Label2->Top = QualityBox->Top;
		 HideTypeBox->Top = Label2->Top;
		 Phrase->Top = Label2->Top + 20;
		 OkBtn->Top = Phrase->Top + Phrase->Height + 7;
		 CancelBtn->Top = OkBtn->Top;
		 CancelBtn->Left = 270;
		 OkBtn->Left = 190;
		 Height = 187;
		 Width = 365;
		 break;
	  case dbpPermission:
		 Caption = " Secure Mode Permission ";
		 Label2->Caption =
			"In order to quit Secure Mode or change the passphrase you need to "
			"enter the current passphrase now";
		 QualityBox->Hide();
		 Cnfirm->Hide();
		 Label3->Hide();
		 //HelpBtn->Hide();
		 Bevel1->Hide();
		 Label2->Top = QualityBox->Top;
		 Phrase->Top = Label2->Top + 65;
		 HideTypeBox->Top = Phrase->Top - 20;
		 OkBtn->Top = Phrase->Top + Phrase->Height + 7;
		 CancelBtn->Top = OkBtn->Top;
		 CancelBtn->Left = 270;
		 OkBtn->Left = 192;
		 ClearBox->Top = OkBtn->Top;
		 ClearBox->Show();
		 Height = 230;
		 Width = 365;
		 break;
	  case dbpChange:
		 Caption = " Secure Mode Change Passphrase ";
		 Label2->Caption = "&Enter new passphrase :";
		 break;
	  case dbpEncrypt:
		 Caption = " Secure Mode ";
		 break;
   }
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::ClearText()
{
   ProgressBar1->Position = 0;
   ZeroMemory(currentpassphrase.c_str(), currentpassphrase.Length());
   currentpassphrase = "";
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::FormCloseQuery(TObject *Sender,
	  bool &CanClose)
{
   if (ModalResult == mrOk)
   {
	  //TIniFile *ini;
	  switch (mode)
	  {
		 case dbpEncrypt:
		 case dbpChange:
			if (Phrase->Text.IsEmpty())
			{
			   MsgBox(this, "Enter a passphrase or click Cancel!   ", MT_INFO, MB_OK);
			   ActiveControl = Phrase;
			   CanClose = false;
			   return;
			}
			if (Phrase->Text.Length() < minpasslen)
			{
			   MsgBox(this,
				 "Your passphrase needs to be at least " + IntToStr(minpasslen) +
				 " characters long!   ",
				  MT_INFO, MB_OK);
			   ActiveControl = Phrase;
			   CanClose = false;
			   ClearText();
			   return;
			}
			if (Cnfirm->Text != Phrase->Text)
			{
			   MsgBox(this, "Your passphrase entries don't match.   ",
				  MT_INFO, MB_OK);
			   ActiveControl = Phrase;
			   CanClose = false;
			   ClearText();
			   return;
			}
			break;
		 case dbpDecrypt:
			if (Phrase->Text.IsEmpty())
			{
			   MsgBox(this, "Enter a passphrase or click Cancel!   ", MT_INFO, MB_OK);
			   ActiveControl = Phrase;
			   CanClose = false;
			   ClearText();
			   return;
			}
			break;
		 case dbpPermission:
			if (Phrase->Text.IsEmpty())
			{
			   MsgBox(this, "Enter a passphrase or click Cancel!   ", MT_INFO, MB_OK);
			   ActiveControl = Phrase;
			   CanClose = false;
			   return;
			}
			if (Phrase->Text != currentpassphrase)
			{
			   MsgBox(this, "Bad passphrase!        ", MT_INFO, MB_OK);
			   attempts++;
			   if (attempts < 3)
			   {
				  ActiveControl = Phrase;
				  CanClose = false;
				  ClearText();
				  return;
			   }
			   ModalResult = mrCancel;
			   return;
			}
			if (ClearBox->Checked)
			{
			   switch (MsgBox(this,
				  "IMPORTANT!\n"
				  "I'm going to CANCEL Secure Mode. Your database will no longer   \n"
				  "be encrypted between sessions. Is that what you want?   ",
				  MT_CONFIRM, MB_YESNO))
			   {
				  case IDNO:
					 CanClose = false;
					 ActiveControl = Phrase;
					 ClearText();
					 break;
				  case IDYES:
					 ModalResult = mrAbort;
					 ClearText();
					 break;
			   }
			   return;
			}
			break;
	  }
	  currentpassphrase = Phrase->Text;
   }
}
//---------------------------------------------------------------------------
// This function is not written by me (Richard). I swiped it from PGP source code

#define NUMRANGES	8
UINT PGPcomdlgPassphraseQuality(LPSTR szPhrase)
{
	UINT quality;
	unsigned short charIndex, rangeIndex, span;
	typedef struct ScoreStruct
	{
		unsigned char firstChar;
		unsigned char lastChar;
	} Range;
	static Range sRanges[] =
	{
		{ 0,		' ' - 1},
		{ ' ',		'A' - 1},
		{ 'A',		'Z'},
		{ 'Z' + 1,	'a' - 1},
		{ 'a',		'z'},
		{ '0',		'9'},
		{ 'z' + 1,	127},
		{ 128,		255},
	};
	BOOL haveRange[NUMRANGES];

        memset(haveRange,'\0',sizeof(haveRange));
        //pgpClearMemory (haveRange, sizeof (haveRange));
	for (charIndex = 1; charIndex < lstrlen (szPhrase); ++charIndex) {
		for (rangeIndex = 0; rangeIndex < NUMRANGES; ++rangeIndex)	{
			UCHAR			theChar;
			Range *			theRange;

			theChar		= szPhrase[charIndex];
			theRange	= &sRanges[rangeIndex];

			if (theChar >= theRange->firstChar &&
				theChar <= theRange->lastChar)
			{
				haveRange[rangeIndex] = TRUE;
				break;
			}
		}
	}

	// determine the total span
	span = 0;
	for (rangeIndex = 0; rangeIndex < NUMRANGES; ++rangeIndex) {
		if (haveRange[rangeIndex]) {
			span += (unsigned short)(1+(sRanges[rangeIndex].lastChar-
				sRanges[rangeIndex].firstChar));
		}
	}

	if (span != 0) {
		double	bitsPerChar	= log(span) / log (2.0);  // base 2 log of span
		ULONG	totalBits	= (ULONG)(bitsPerChar * lstrlen (szPhrase));

		// assume a maximum quality of 128 bits
		quality = (totalBits * 100) / 128;
		if (quality > 100) quality = 100;
	}
	else {
		quality = 0;
	}

	return (quality);
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::PhraseChange(TObject *Sender)
{
   ProgressBar1->Position = (WORD)PGPcomdlgPassphraseQuality(Phrase->Text.c_str());
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::HideTypeBoxClick(TObject *Sender)
{
   Phrase->Font->Color = HideTypeBox->Checked ? clWindow : clBlack;
   Cnfirm->Font->Color = Phrase->Font->Color;
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::FormShow(TObject *Sender)
{
   HideTypeBoxClick(HideTypeBox);
}

//---------------------------------------------------------------------------
// For Session Keys
//---------------------------------------------------------------------------

AnsiString __fastcall Base64EncodeData(char *buffer, int length)
{
   const char Base64Chars[] =
   {
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
	'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
    'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/', '='
   };

   char DataIn[3];
   char DataOut[81];
   AnsiString encodedline;
   int i, Count = 0, BytesRd, InBytes = sizeof(DataIn);

   TMemoryStream *f = new TMemoryStream();
   f->Write(buffer, length);
   f->Position = 0;

   while (true)
   {
      BytesRd = f->Read(DataIn, InBytes);
	  for (i = 1; i < 4; i++)
         DataOut[Count + i] = 0x40;

      DataOut[Count]   = char((DataIn[0] & 0xFC) >> 2);
      DataOut[Count + 1] = char((DataIn[0] & 0x03) << 4);
      if (BytesRd > 1)
      {
         DataOut[Count + 1] += char((DataIn[1] & 0xF0) >> 4);
         DataOut[Count + 2]  = char((DataIn[1] & 0x0F) << 2);
         if (BytesRd > 2)
         {
            DataOut[Count + 2] += char((DataIn[2] & 0xC0) >> 6);
            DataOut[Count + 3]  =  char(DataIn[2] & 0x3F);
         }
	  }

	  for (i = 0; i < 4; i++)
         DataOut[Count + i] = byte(Base64Chars[DataOut[Count + i]]);

	  Count += 4;
      if (Count < 60 && BytesRd == InBytes)
         continue;

      DataOut[Count] = '\0';

      encodedline += AnsiString(&DataOut[0]);

      if (BytesRd < InBytes)
         break;

	  Count = 0;
   }

   delete f;
   return encodedline;
}
//---------------------------------------------------------------------------

