/*
*  secdbs.h 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#ifndef secdbsH
#define secdbsH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ComCtrls.hpp>
#include "dbs.h"
#include <ExtCtrls.hpp>
class TDBSecModeDlg;
class TSecureDB;
//---------------------------------------------------------------------------

TSecureDB* __fastcall OpenSecureDB(const AnsiString &FileName);
bool __fastcall CloseSecureDB(TSecureDB *DB);

class TSecureDB : public TDB
{
   TDBSecModeDlg *DBSecModeDlg;
   TMemoryStream* __fastcall GetMemoryStream();
public:
   __property TMemoryStream *memstream = { read = GetMemoryStream };

   TMemoryStream *MasterPassword;

   __fastcall virtual TSecureDB(const AnsiString &FileName, bool InMemory);
   __fastcall virtual TSecureDB(const AnsiString &FileName);
   __fastcall virtual~TSecureDB();
   AnsiString __fastcall Name();
   // secure mode
   bool __fastcall OpenDatabase();
   bool __fastcall CloseDatabase();
   bool __fastcall IsSecureMode();
   bool __fastcall EncryptDatabase(AnsiString &passphrase);
   bool __fastcall DecryptDatabase(AnsiString &passphrase);
   //bool __fastcall CreateBackupDB();
   bool __fastcall Save();
   bool __fastcall ManageSecureMode(TForm *Owner);
   int  __fastcall PermissionSecureMode(TForm *Owner);
   bool __fastcall GetUserPassphrase(TForm *Owner, AnsiString &passphrase);
   // settings
   bool __fastcall LoadSettings(TStrings *settings);
   bool __fastcall SaveSettings(TStrings *settings);
   AnsiString __fastcall LoadSetting(const AnsiString &Ident, const AnsiString &Default);
   bool __fastcall SaveSetting(const AnsiString &Ident, const AnsiString &Value);
   // session key
   bool __fastcall CreateSessionKey();
   bool __fastcall SaveSessionKey(AnsiString &key); // used during shutdown
   bool __fastcall LoadSessionKey(AnsiString &key);
   bool __fastcall ClearSessionKey();
   // config
   bool __fastcall LoadConfig(const AnsiString &section, TStrings *config);
   bool __fastcall SaveConfig(const AnsiString &section, TStrings *config);
 //void __fastcall ShowStatus(const HANDLE handle, const AnsiString &msg);
   void __fastcall ShowStatus(const AnsiString &msg);
};


class TDBSecModeDlg : public TForm
{
__published:	// IDE-managed Components
		TGroupBox *QualityBox;
		TProgressBar *ProgressBar1;
		TLabel *Label2;
		TCheckBox *HideTypeBox;
		TMemo *Phrase;
		TLabel *Label3;
		TMemo *Cnfirm;
		TButton *OkBtn;
		TButton *CancelBtn;
		TCheckBox *ClearBox;
	TBevel *Bevel1;
		void __fastcall FormCloseQuery(TObject *Sender, bool &CanClose);
		void __fastcall PhraseChange(TObject *Sender);
		void __fastcall HideTypeBoxClick(TObject *Sender);
		void __fastcall FormShow(TObject *Sender);
private:	// User declarations
public:		// User declarations
		int mode, attempts, minpasslen;
		AnsiString currentpassphrase;
		__fastcall virtual TDBSecModeDlg(TComponent* Owner);
		__fastcall virtual~TDBSecModeDlg();
		void __fastcall SetMode(int amode);
		void __fastcall ClearText();
};

const AnsiString Config        = "settings";
//const AnsiString PGPConfig     = "pgp";
const AnsiString ERASConfig    = "eraser";
const AnsiString TheSession    = "session";
const AnsiString Key           = "key";

bool __fastcall MessageFlagIsSet(AnsiString key, WORD flag);
void __fastcall UnSetMessageFlag(AnsiString &key, WORD flag);
void __fastcall SetMessageFlag(AnsiString &key, WORD flag);

// password dialog mode
int const dbpEncrypt = 1;
int const dbpDecrypt = 2;
int const dbpChange  = 3;
int const dbpPermission = 4;  // allow changes or removal of passphrase

UINT PGPcomdlgPassphraseQuality(LPSTR szPhrase);
AnsiString __fastcall Base64EncodeData(char *buffer, int length);
//---------------------------------------------------------------------------
extern PACKAGE TSecureDB *SecureDB;
//extern PACKAGE TDBSecModeDlg *DBSecModeDlg;
//--------------------------------------------------------------------------
#endif
