/*
*  subjectsdlgs.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "subjectsdlgs.h"
#include "global.h"
#include "utility.h"
#include "dsadlgs.h"
#include "gpg4.h"
#include "qsadbs.h"
#include "help.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TSubjectsDlg *SubjectsDlg;
//---------------------------------------------------------------------------

bool __fastcall CreateSubjectPassphrasePair(TForm *Owner, AnsiString &SubAndPass, TStrings *Existing)
{
   SubAndPass = "";
   TSubjectsDlg *subsdlg = new TSubjectsDlg(Owner);
   subsdlg->SetSubAndPass("", Existing);
   subsdlg->ShowModal();
   bool ok = subsdlg->ModalResult == mrOk;
   if (ok)
   {
	  SubAndPass = subsdlg->GetSubAndPass();
   }
   delete subsdlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall EditSubjectPassphrasePair(TForm *Owner, AnsiString &SubAndPass, TStrings *Existing)
{
   TSubjectsDlg *subsdlg = new TSubjectsDlg(Owner);
   subsdlg->SetSubAndPass(SubAndPass, Existing);
   //subsdlg->ActiveControl = subsdlg->PGPKey;
   subsdlg->ShowModal();
   bool ok = subsdlg->ModalResult == mrOk;
   if (ok)
   {
	  SubAndPass = subsdlg->GetSubAndPass();
   }
   delete subsdlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TSubjectsDlg::TSubjectsDlg(TComponent* Owner)
	: TForm(Owner)
{
   Subject->Clear();
   SubPhrase->Clear();
   KeyPhrase->Clear();
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Subject->Font, ini->ReadString("settings", "windowfont", OptsDefWindowFont));
   AssignFont(SubPhrase->Font, ini->ReadString("settings", "editorfont", OptsDefEditorFont));
   AssignFont(KeyPhrase->Font, ini->ReadString("settings", "editorfont", OptsDefEditorFont));
   EditColor = SubPhrase->Font->Color;
   SubPhrase->Font->Color = clWindow;
   KeyPhrase->Font->Color = clWindow;
   delete ini;
   existing = new TStringList();
   data = new TStringList();
   ActiveControl = PGPKey;
   Active->Checked = true;
}
//---------------------------------------------------------------------------

__fastcall TSubjectsDlg::~TSubjectsDlg()
{
   delete data;
   delete existing;
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult == mrCancel)
   {
	  if (Modified())
	  {
		 switch(MsgBox(this,
			"Your data is modified. Save your changes?   ",
			 MT_CONFIRM, MB_YESNOCANCEL))
		 {
			case IDCANCEL: CanClose = false;
			case IDNO: return;
		 }
		 ModalResult = mrOk;
	  }
   }

   if (ModalResult == mrOk)
   {
	  AnsiString ephrase = EncryptSub->Checked ? SubPhrase->Text : AnsiString("");
	  if (PGPKey->ItemIndex == 0)
	  {
		 MsgBox(this,
		   "The NYM field is required!   ",
			MT_INFO, MB_OK);
		 CanClose = false;
		 return;
	  }
	  if (Subject->Text.IsEmpty())
	  {
		 MsgBox(this,
		   "The subject field is required!   ",
			MT_INFO, MB_OK);
		 CanClose = false;
		 return;
	  }
	  if (EncryptSub->Checked && ephrase.IsEmpty())
	  {
		 MsgBox(this,
		   "With encryption enabled a password is required!   ",
			MT_INFO, MB_OK);
		 CanClose = false;
		 return;
	  }
	  if (EncryptKey->Checked)
	  {
		 AnsiString kphrase = KeyPhrase->Text;
		 if (kphrase.IsEmpty())
		 {
			MsgBox(this,
			  "With encrypt-key enabled a password is required!   ",
			   MT_INFO, MB_OK);
			CanClose = false;
			return;
		 }
		 if (kphrase.Pos(",") > 0)
		 {
			if (DSABox(this, "commas",
			   "I see your Encrypt-Key password contains a comma. In case "
			   "you didn't know, commas are reserved for separating multiple "
			   "passwords in the Encrypt-Key field. If you have a single "
			   "password that contains a comma, the decryption will fail.",
			   "Encrypt-Key Comma Usage",
				MB_OKCANCEL) == IDCANCEL)
			{
			   CanClose = false;
			   return;
			}
		 }
	  }
	  if (SubjectExists(Subject->Text, ephrase))
	  {
		 if (ephrase.IsEmpty())
		 {
			MsgBox(this,
			  "This HASHED subject already exists!   ",
			   MT_INFO, MB_OK);
		 }
		 else
		 {
			MsgBox(this,
			  "This ESUB subject/password pair already exists!   ",
			   MT_INFO, MB_OK);
		 }
		 CanClose = false;
		 return;
	  }
   }
}
//---------------------------------------------------------------------------

bool __fastcall TSubjectsDlg::SubjectExists(const AnsiString &Subject, const AnsiString &EPhrase)
{
//look(existing);
   bool exists = false;
   AnsiString ephrase, subject;

   TStringList *item = new TStringList();
   for (int i = 0; i < existing->Count; i++)
   {
	  item->CommaText = existing->Strings[i];
	  subject = item->Values["subject"];
	  ephrase = item->Values["esub"] == "1" ? item->Values["ephrase"] : AnsiString("");

	  if (subject == Subject)
	  {
		 if (EPhrase.IsEmpty() && ephrase.IsEmpty())
		 {
			exists = true;
			break;
		 }
		 if (EPhrase == ephrase)
		 {
			exists = true;
			break;
		 }
	  }
   }
   delete item;

   return exists;
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::SetSubAndPass(const AnsiString &SubAndPass, TStrings *Existing)
{
   subandpass = SubAndPass;
   data->CommaText = subandpass;
   subno = data->Values["subno"];
   if (!SubAndPass.IsEmpty())
	  Active->Checked = data->Values["active"] == "1";
   Subject->Text = data->Values["subject"];
   EncryptSub->Checked = data->Values["esub"] == "1";
   SubPhrase->Text = data->Values["ephrase"];
   EncryptKey->Checked = data->Values["ekey"] == "1";
   KeyPhrase->Text = data->Values["kphrase"];
   LoadKeys(data->Values["keyid"]);
   TStringList *item = new TStringList();
   for (int i = 0; i < Existing->Count; i++)
   {
	  item->CommaText = Existing->Strings[i];
	  if (EncryptSub->Checked)
	  {
		 if (Subject->Text != item->Values["subject"] || SubPhrase->Text != item->Values["ephrase"])
			existing->Add(Existing->Strings[i]);
		 continue;
	  }
	  if (Subject->Text != item->Values["subject"])
		 existing->Add(Existing->Strings[i]);
   }
   if (!Active->Checked)
   {
      ActiveClick(NULL);
   }
   delete item;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TSubjectsDlg::GetSubAndPass()
{
   data->Clear();
   if (subno.IsEmpty())
   {
	  int current = DB->LoadSetting("nextsubject", "2000").ToIntDef(2000);
	  if (current < 2000)
		 current = 2000;
	  //assert(current != 0);
	  subno = IntToStr(current);
	  DB->SaveSetting("nextsubject", IntToStr(++current));
   }
   data->Values["subno"] = subno;
   data->Values["active"] = Active->Checked ? "1":"0";
   data->Values["subject"] = Subject->Text;
   data->Values["esub"] = EncryptSub->Checked ? "1":"0";
   data->Values["ephrase"] = SubPhrase->Text;
   data->Values["ekey"] = EncryptKey->Checked ? "1":"0";
   data->Values["kphrase"] = KeyPhrase->Text;
   data->Values["keyid"] = GetSelectedKeyID();
   return data->CommaText;
 }
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::SubjectChange(TObject *Sender)
{
   OkBtn->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::PhraseChange(TObject *Sender)
{
   OkBtn->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::HideBox1Click(TObject *Sender)
{
   if (!HideBox1->Checked)
	  SubPhrase->PasswordChar = '\0';
   else
	  SubPhrase->PasswordChar = '*';
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::HideBox2Click(TObject *Sender)
{
   if (!HideBox2->Checked)
	  KeyPhrase->PasswordChar = '\0';
   else
	  KeyPhrase->PasswordChar = '*';
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::EncryptSubClick(TObject *Sender)
{
   if (EncryptSub->Checked)
   {
	  Label2->Enabled = true;
	  Label7->Enabled = true;
	  SubPhrase->Enabled = true;
	  SubPhrase->Color = clWindow;
	  SubPhrase->Font->Color = EditColor;
	  Label1->Caption = "subject will be encrypted";
	  Label3->Caption = "&Subject (ESUB)";
	  ActiveControl = SubPhrase;
	  HideBox1->Enabled = true;
   }
   else
   {
	  Label2->Enabled = false;
	  Label2->Enabled = false;
	  SubPhrase->Enabled = false;
	  SubPhrase->ParentColor = true;
	  Label1->Caption = "subject will be hashed";
	  Label3->Caption = "&Subject (HSUB)";
	  ActiveControl = Subject;
	  HideBox1->Enabled = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::EncryptKeyClick(TObject *Sender)
{
   if (EncryptKey->Checked)
   {
	  Label5->Enabled = true;
	  Label6->Enabled = true;
	  KeyPhrase->Enabled = true;
	  KeyPhrase->Color = clWindow;
	  KeyPhrase->Font->Color = EditColor;
	  ActiveControl = KeyPhrase;
	  HideBox2->Enabled = true;
   }
   else
   {
	  Label5->Enabled = false;
	  Label6->Enabled = false;
	  KeyPhrase->Enabled = false;
	  KeyPhrase->ParentColor = true;
	  ActiveControl = Subject;
	  HideBox2->Enabled = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::LoadKeys(const AnsiString &KeyID)
{
   PGPKey->Items->Clear();
   PGPKey->Items->BeginUpdate();

   AnsiString keyid, key, keyuid;
   TStringList *keys = new TStringList();
   GPGListSecretKeysWithColons(keys, "", true);

   int pos = 0, defkeypos = -1;
   PGPKey->Items->Add("Select a PGP key");
   for (int i = 0; i < keys->Count; i++)
   {
	  key = keys->Strings[i];
	  if (GetTok(lkfKeyValid, key, ':') == "e" || GetTok(lkfKeyType, key, ':') != "sec")
		 continue;

	  keyid = GetTok(lkfKeyID, key, ':');
	  keyid = "0x"+keyid.SubString(keyid.Length() - 7, 8);
	  keyuid = GetTok(lkfKeyUID, key, ':');
	  PGPKey->Items->Add(keyid+"  "+keyuid);
	  pos++;

	  if (keyid == KeyID)
	  {
		 orgkeyid = KeyID;
		 defkeypos = pos;
	  }
   }

   if (defkeypos < 0 && KeyID != "")
   {
	  MsgBox(this,
		"PROBLEM: The PGP key, "+KeyID+", for this nym is missing!   ",
		 MT_ERROR, MB_OK);
	  PGPKey->ItemIndex = 0;
   }

   PGPKey->ItemIndex = defkeypos < 0 ? 0 : defkeypos;

   delete keys;
   PGPKey->Items->EndUpdate();
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::FormShow(TObject *Sender)
{
   orgactive = Active->Checked;
   orgencrypt = EncryptSub->Checked;
   orgsubject = Subject->Text;
   orgsubphrase = SubPhrase->Text;
   orgkeyphrase = KeyPhrase->Text;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TSubjectsDlg::GetSelectedKeyID()
{
   AnsiString keyid;
   if (PGPKey->ItemIndex > 0)
	  keyid = GetTok(0, PGPKey->Text, ' ');
   return keyid;
}
//---------------------------------------------------------------------------

bool __fastcall TSubjectsDlg::Modified()
{
   return Active->Checked != orgactive || EncryptSub->Checked != orgencrypt ||
	  SubPhrase->Text != orgsubphrase || KeyPhrase->Text != orgkeyphrase ||
	  Subject->Text != orgsubject || GetSelectedKeyID() != orgkeyid;
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::SubPhraseEnter(TObject *Sender)
{
   SubPhrase->SelStart = SubPhrase->Text.Length();
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::KeyPhraseEnter(TObject *Sender)
{
   KeyPhrase->SelStart = KeyPhrase->Text.Length();
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::ActiveClick(TObject *Sender)
{
   bool active = Active->Checked;
   Label8->Enabled = active;
   PGPKey->Enabled = active;
   Label3->Enabled = active;
   Label1->Enabled = active;
   Subject->Enabled = active;
   EncryptSub->Enabled = active;
   Label2->Enabled = active;
   Label7->Enabled = active;
   HideBox1->Enabled = active;
   SubPhrase->Enabled = active;
   EncryptKey->Enabled = active;
   Label6->Enabled = active;
   Label5->Enabled = active;
   HideBox2->Enabled = active;
   KeyPhrase->Enabled = active;
   if (active)
   {
	  Subject->Color = clWindow;
	  if (EncryptSub->Checked)
	  {
		 SubPhrase->Color = clWindow;
		 Label2->Enabled = true;
		 Label7->Enabled = true;
	  }
	  else
	  {
		 SubPhrase->ParentColor = true;
		 Label2->Enabled = false;
		 Label7->Enabled = false;
	  }
	  if (EncryptKey->Checked)
	  {
		 KeyPhrase->Color = clWindow;
		 Label5->Enabled = true;
		 Label6->Enabled = true;
	  }
	  else
	  {
		 KeyPhrase->ParentColor = true;
		 Label5->Enabled = false;
		 Label6->Enabled = false;
	  }
   }
   else
   {
	  Subject->ParentColor = true;
	  SubPhrase->ParentColor = true;
	  KeyPhrase->ParentColor = true;
	  Label2->Enabled = false;
	  Label5->Enabled = false;
	  Label6->Enabled = false;
	  Label7->Enabled = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsDlg::HelpBtnClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/subjectlines.htm>newshelp");
}
//---------------------------------------------------------------------------

